/*
 *  Copyright (C) 2003  Clive Butler
 *
 *  Author's email address: cebdev@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <classlib/String.hpp>

#include "BrowserConfiguration.hpp"
#include "BrowserSnmpProfile.hpp"
#include "BrowserNode.hpp"

BrowserConfiguration::BrowserConfiguration()
               : profiles(0), nodes(0), preferSystemIcons(false),
                 textFontFamily(0), textFontSize(0),
                 applicationFontFamily(0), applicationFontSize(0) {
  profiles = new ObjectList();
  nodes = new ObjectList();
  }

BrowserConfiguration::BrowserConfiguration(const BrowserConfiguration& other) {
  profiles = new ObjectList();
  ListIterator profileIterator = other.profiles -> iterator();
  while (profileIterator.hasNext()) {
    BrowserSnmpProfile *profile = (BrowserSnmpProfile *)profileIterator.next();
    profiles -> add(new BrowserSnmpProfile(*profile));
    }

  nodes = new ObjectList();
  ListIterator nodeIterator = other.nodes -> iterator();
  while (nodeIterator.hasNext()) {
    BrowserNode *node = (BrowserNode *)nodeIterator.next();
    nodes -> add(new BrowserNode(*node));
    }

  preferSystemIcons = other.preferSystemIcons;

  if (other.applicationFontFamily)
    applicationFontFamily = new String(other.applicationFontFamily);
  else
    applicationFontFamily = 0;

  applicationFontSize = other.applicationFontSize;

  if (other.textFontFamily)
    textFontFamily = new String(other.textFontFamily);
  else
    textFontFamily = 0;

  textFontSize = other.textFontSize;
  }

BrowserConfiguration::~BrowserConfiguration() {
  delete profiles;
  delete nodes;
  delete textFontFamily;
  delete applicationFontFamily;
  }
