/*
 *  Copyright (C) 2003  Clive Butler
 *
 *  Author's email address: cebdev@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef BROWSERGROUPVIEWLAYOUT_H
#define BROWSERGROUPVIEWLAYOUT_H

#include "BrowserGroupViewItemWidget.hpp"

#include <qlist.h>
#include <qabstractlayout.h>

class QLabel;
class QColor;

class BrowserGroupViewLayoutItem : public QLayoutItem {
  public:
    BrowserGroupViewLayoutItem(QWidget *parent,const char *name,
                               const char *value, int widthGoal);
    QSize sizeHint() const;
    QSize minimumSize() const;
    QSize maximumSize() const;
    QSizePolicy::ExpandData expanding() const;
    void setGeometry(const QRect&);
    QRect geometry() const;
    bool isEmpty() const;

    void move(int x,int y);
    void setBackgroundColor(const QColor& backgroundColor);
    void setWidths(int nameColumnWidth,int valueColumnWidth);
    void setSize(int width,int height);
    int height() const;
    int width() const;
    int nameWidth() const;
    int separatorWidth() const;
    int valueWidth() const;

    BrowserGroupViewItemWidget *widget;
  };

class BrowserGroupViewLayoutData;

class BrowserGroupViewLayout : public QLayout {
  Q_OBJECT

  public:
    BrowserGroupViewLayout(QWidget *parent);
    ~BrowserGroupViewLayout();

    void addItem(QLayoutItem *item);
    QSize sizeHint() const;
    QSize minimumSize() const;
    QLayoutIterator iterator();
    void setGeometry(const QRect &rect);

    void visibleSizeChange(const QSize& size);

    //
    // The following block are optional re-implementations
    //
    QLayout::ResizeMode resizeMode() const;
    bool hasHeightForWidth () const;
    int heightForWidth (int width) const;
    QSizePolicy::ExpandData expanding() const;

  private:
    void calculateColumns(int columns) const;
    void calculateOptimumColumns(int width) const;

    BrowserGroupViewLayoutData *layoutData;

  private slots:
    void redraw();
  };

#endif
