/*
 *  Copyright (C) 2003  Clive Butler
 *
 *  Author's email address: cebdev@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "BrowserProfileListViewItem.hpp"

BrowserProfileListViewItem::BrowserProfileListViewItem(
                QListView *parent,const BrowserSnmpProfile *profile)
          : QListViewItem(parent), profile(profile) {

  refreshProfile();
  }

BrowserProfileListViewItem::~BrowserProfileListViewItem() {
  }

BrowserProfileListViewItem::BrowserProfileListViewItem(
               const BrowserProfileListViewItem& other)
          : QListViewItem(other) {
  profile = other.profile;
  refreshProfile();
  }

BrowserProfileListViewItem& BrowserProfileListViewItem::operator=(
               const BrowserProfileListViewItem& other) {
  profile = other.profile;
  refreshProfile();

  return *this;
  }

void BrowserProfileListViewItem::refreshProfile() {
  //
  // Populate the list view item from this profile. Note that this
  // code depends on the column ordering matching the QtDesigner
  // form. TBD - support user preference saving for column order
  //
  QString string;

  string = profile -> getName();
  setText(0,string);

  switch (profile -> getVersion()) {
    case BrowserSnmpProfile::v1:
      string = "v1";
      setText(2,"Community");
      break;
    case BrowserSnmpProfile::v2c:
      string = "v2c";
      setText(2,"Community");
      break;
    case BrowserSnmpProfile::v3:
      string = "v3";

      if (profile -> getAuthProtocol() == BrowserSnmpProfile::authNone)
        setText(2,"User");
      else if (profile -> getPrivProtocol() == BrowserSnmpProfile::privNone)
        setText(2,"User Auth");
      else
        setText(2,"User Auth & Priv");

      break;
    }
  setText(1,string);
  }

const BrowserSnmpProfile *BrowserProfileListViewItem::getProfile() const {
  return profile;
  }
