/*
 *  Copyright (C) 2003  Clive Butler
 *
 *  Author's email address: cebdev@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <cstdio>
#include <cstring>
#include <cstdlib>

#include <classlib/Xml.hpp>

#include "BrowserReportFunctionGraphLoader.hpp"
#include "BrowserReportFunctionParameter.hpp"
#include "BrowserReportFunctionParametersLoader.hpp"

BrowserReportFunctionGraphLoader::BrowserReportFunctionGraphLoader(
                                                    const Xml& xml)
                 : XmlLoader(xml), function(0) {
  }

BrowserReportFunctionGraphLoader::~BrowserReportFunctionGraphLoader() {
  delete function;
  }

BrowserReportFunctionGraphEntry *
           BrowserReportFunctionGraphLoader::parseGraphEntry(
                                                 const Xml& xml,
                                                 XmlElement entryElement) {
  const char *name = 0;
  const char *objectName = 0;
  int sample = 0;
  int attributeCount = xml.getElementAttributeCount(entryElement);
  for (int a = 0 ; a < attributeCount ; a++ ) {
    const char *attributeName = xml.getElementAttributeName(entryElement,a);
    const char *attributeValue = xml.getElementAttributeValue(entryElement,a);
    if (strcmp(attributeName,"name") == 0) name = attributeValue;
    else if (strcmp(attributeName,"object") == 0) objectName = attributeValue;
    else if (strcmp(attributeName,"sample") == 0)
      sample = std::atoi(attributeValue);
    else {
      THROW(new BrowserException("Unrecognised attribute",
                                 "Graph entry has 'name', 'object' "
                                 "and 'sample' attributes"),0);
      }
    }

  if ((! name) || (! objectName)) {
    THROW(new BrowserException("Missing attribute",
                               "Graph entry requires 'name' "
                               "and 'object' attributes"),0);
    }

  if (sample)
    return new BrowserReportFunctionGraphEntry(name,objectName,sample);
  else
    return new BrowserReportFunctionGraphEntry(name,objectName);
  }

ObjectList *
         BrowserReportFunctionGraphLoader::parseGraphEntryList(
                                                  const Xml& xml,
                                                  XmlElement entriesElement) {

  ObjectList *entries = new ObjectList();

  int childCount = xml.getElementChildCount(entriesElement);
  for (int i = 0 ; i < childCount ; i++ ) {
    XmlElement element = xml.getElementChild(entriesElement,i);
    const char *elementName = xml.getElementName(element);
    if (strcmp(elementName,"entry") == 0) {
      BrowserReportFunctionGraphEntry *entry = parseGraphEntry(xml,element);
      if (exception) {
        delete entries;
        return 0;
        }

      entries -> add(entry);
      }
    }

  return entries;
  }

void BrowserReportFunctionGraphLoader::load(XmlElement functionElement) {

  function = new BrowserReportFunctionGraph();

  int childCount = xml.getElementChildCount(functionElement);
  if (childCount) {
    for (int i = 0 ; i < childCount ; i++ ) {
      XmlElement childElement = xml.getElementChild(functionElement,i);
      const char *childElementName = xml.getElementName(childElement);
      if (std::strcmp(childElementName,"parameters") == 0) {
        BrowserReportFunctionParametersLoader loader(xml);
        loader.load(childElement);
        if (loader.exception) {
          RETHROW(loader.exception,);
          }

        function -> setParameterList(loader.takeParameterList());
        }
      else if (std::strcmp(childElementName,"entries") == 0) {
        ObjectList *entries = parseGraphEntryList(xml,childElement);
        if (exception) {
          return;
          }

        function -> setEntryList(entries);
        }
      else {
        char diagnostic[128];
        std::sprintf(diagnostic,"Unrecognised element '%s' in Graph function",
                     childElementName);
        THROW(new BrowserException("Error in report definition",diagnostic),);
        }
      }
    }
  }

BrowserReportFunction *BrowserReportFunctionGraphLoader::takeFunction() {

  BrowserReportFunction *returnedFunction = (BrowserReportFunction *)function;
  function = 0;
  return returnedFunction;
  }

