/*
 *  Copyright (C) 2003  Clive Butler
 *
 *  Author's email address: cebdev@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <cstdio>
#include <cstring>

#include <classlib/Xml.hpp>

#include "BrowserReportFunctionWalkLoader.hpp"
#include "BrowserReportFunctionParameter.hpp"
#include "BrowserReportFunctionParametersLoader.hpp"

BrowserReportFunctionWalkLoader::BrowserReportFunctionWalkLoader(const Xml& xml)
                 : XmlLoader(xml), function(0) {
  }

BrowserReportFunctionWalkLoader::~BrowserReportFunctionWalkLoader() {
  delete function;
  }

void BrowserReportFunctionWalkLoader::load(XmlElement functionElement) {

  function = new BrowserReportFunctionWalk();

  //
  // See if the parameter value is supplied as an attribute
  //
  int attributeCount = xml.getElementAttributeCount(functionElement);
  if (attributeCount == 2) {
    BrowserReportFunctionParameter *parameter =
                         new BrowserReportFunctionParameter("walkStart");
    parameter -> setValue(xml.getElementAttributeValue(functionElement,1));
    ObjectList *parameterList = new ObjectList();
    parameterList -> add(parameter);
    function -> setParameterList(parameterList);
    }
  else {
    int childCount = xml.getElementChildCount(functionElement);
    if (childCount) {
      for (int i = 0 ; i < childCount ; i++ ) {
        XmlElement childElement = xml.getElementChild(functionElement,i);
        const char *childElementName = xml.getElementName(childElement);
        if (std::strcmp(childElementName,"parameters") == 0) {
          BrowserReportFunctionParametersLoader loader(xml);
          loader.load(childElement);
          if (loader.exception) {
            RETHROW(loader.exception,);
            }

          function -> setParameterList(loader.takeParameterList());
          }
        else {
          char diagnostic[128];
          std::sprintf(diagnostic,"Unrecognised element '%s' in Walk function",
                       childElementName);
          THROW(new BrowserException("Error in report definition",diagnostic),);
          }
        }
      }
    }
  }

BrowserReportFunction *BrowserReportFunctionWalkLoader::takeFunction() {

  BrowserReportFunction *returnedFunction = (BrowserReportFunction *)function;
  function = 0;
  return returnedFunction;
  }

