/*
 *  Copyright (C) 2003  Clive Butler
 *
 *  Author's email address: cebdev@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <cstdio>
#include <cstdlib>

#ifdef QWS
#include <qpe/qpeapplication.h>
#include <qpe/config.h>
#include <qpe/global.h>
#else
#include <myqpe/Config.hpp>
#include <myqpe/Resource.hpp>
#endif

#include "BrowserResource.hpp"

#ifdef QWS
bool BrowserResource::preferSystemIcons = false;
bool BrowserResource::maximiseApplication = true;
bool BrowserResource::maximiseDialogs = true;
#else
bool BrowserResource::preferSystemIcons = true;
bool BrowserResource::maximiseApplication = false;
bool BrowserResource::maximiseDialogs = false;
#endif
int BrowserResource::bootCount = 0;

void BrowserResource::loadPreferences() {
  Config preferences(applicationName(),Config::User);
  preferences.setGroup("General");
  preferSystemIcons = preferences.readBoolEntry("preferSystemIcons",
                                                preferSystemIcons);

  maximiseApplication = preferences.readBoolEntry("maximiseApplication",
                                                  maximiseApplication);

  maximiseDialogs = preferences.readBoolEntry("maximiseDialogs",
                                              maximiseDialogs);

  bootCount = preferences.readNumEntry("bootCount",bootCount);
  bootCount++ ;
  preferences.writeEntry("bootCount",bootCount);
  }

const char *BrowserResource::applicationName() {
  return "SNMPzBrowser";
  }

const char *BrowserResource::applicationGroupName() {
  return "SNMPz";
  }

const char *BrowserResource::applicationTitle() {
  return "SNMP Browser";
  }

const char *BrowserResource::configurationFileName() {
  return "Configuration.xml";
  }

const char *BrowserResource::databaseFileName() {
  return "smi.db";
  }

const char *BrowserResource::reportsFileName() {
  return "Reports.xml";
  }

const char *BrowserResource::reportsDirName() {
  return "Reports";
  }

QPixmap BrowserResource::loadPixmap(const char *name) {
#ifdef QWS
  //
  // Qtopia has a built-in path (not to mention a built-in set of icons)
  // so the names we give it are always relative
  //
  QString systemPixmapPath = name;

  QString applicationPixmapPath = applicationGroupName();
  applicationPixmapPath = applicationPixmapPath + "/" + name;
#else
  QString systemPixmapPath = "/usr/share/icons/crystalsvg/16x16/actions/";
  systemPixmapPath = systemPixmapPath + name;

  QString applicationPixmapPath = "/opt/";
  applicationPixmapPath = applicationPixmapPath + applicationGroupName() +
                          "/icons/" + name;
#endif

  QPixmap pixmap;
  if (preferSystemIcons) {
    pixmap = Resource::loadPixmap(systemPixmapPath);
    if (! pixmap.isNull()) return pixmap;

    pixmap = Resource::loadPixmap(applicationPixmapPath);
    if (! pixmap.isNull()) return pixmap;
    }
  else {
    pixmap = Resource::loadPixmap(applicationPixmapPath);
    if (! pixmap.isNull()) return pixmap;

    pixmap = Resource::loadPixmap(systemPixmapPath);
    if (! pixmap.isNull()) return pixmap;
    }

  return pixmap;
  }

QString BrowserResource::homeDirPath() {
  QString path = getenv("HOME");
  path += "/";

  return path;
  }

const char *BrowserResource::getUserName() {
  const char *id = getenv("USER");
  if (! id) return "nobody";
  return id;
  }

QString BrowserResource::getApplicationDirUser() {
  QString path = homeDirPath();
#ifdef QWS
  path = path + "Applications/" + applicationName() + "/";
#else
  path = path + "." + applicationName() + "/";
#endif
  return path;
  }

QString BrowserResource::getApplicationGroupDirUser() {
  QString path = homeDirPath();
#ifdef QWS
  path = path + "Applications/" + applicationGroupName() + "/";
#else
  path = path + "." + applicationGroupName() + "/";
#endif
  return path;
  }

QString BrowserResource::getApplicationSharedDir() {
#ifdef QWS
  QString path = QPEApplication::qpeDir() + "etc/" +
                          applicationName() + "/";
#else
  QString path = "/opt/";
  path = path + applicationGroupName() + "/" + applicationName() + "/";
#endif
  return path;
  }
