/*
 *  Copyright (C) 2003  Clive Butler
 *
 *  Author's email address: cebdev@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef BROWSERRESOURCE_H
#define BROWSERRESOURCE_H

#ifdef QWS
#include <qpe/resource.h>
#else
#include <myqpe/Resource.hpp>
#endif

class BrowserResource : public Resource {
  public:
    //
    // Load the named pixmap from one of the places we can load pixmaps
    //
    static QPixmap loadPixmap(const char *name);

    //
    // Load the QPE preferences that affect browser resource behaviour
    //
    static void loadPreferences();

    //
    // Name of this application (no spaces in name, used for pathnames)
    //
    static const char *applicationName();

    //
    // Name of this application's group (no spaces in name, used for
    // pathnames shared with several related applications)
    //
    static const char *applicationGroupName();

    //
    // Title of this application (used in user interfaces). Note that
    // dialog titles may be coded by QtDesigner, so this is not
    // the only place to make changes
    //
    static const char *applicationTitle();

    //
    // The name of the configuration file for this application (the
    // one stored in the user's Applications/<appname> directory)
    // (e.g. "Configuration.xml").
    //
    static const char *configurationFileName();

    //
    // The name of the database file for this application (the
    // one stored in the user's Applications/<appname> directory)
    // (e.g. "smi.db").
    //
    static const char *databaseFileName();

    //
    // The name of the reports definition file for this application (the
    // one stored in the user's Applications/<appname> directory)
    // (e.g. "Reports.xml").
    //
    static const char *reportsFileName();

    //
    // The name of the browser reports directory
    //
    static const char *reportsDirName();

    //
    // Absolute path of this user's Application directory for this
    // application (e.g. "/home/fred/Applications/SNMPBrowser/"). The
    // path includes the trailing /, so callers shouldn't mess
    // with directory separators.
    //
    static QString getApplicationDirUser();

    //
    // Absolute path of this user's Application directory for this
    // application group (e.g. "/home/fred/Applications/SNMPTools/"). The
    // path includes the trailing /, so callers shouldn't mess
    // with directory separators.
    //
    static QString getApplicationGroupDirUser();

    //
    // Absolute path of a read-only directory for this application,
    // shared with all other users of this application
    //
    static QString getApplicationSharedDir();

    //
    // Absolute path of this user's home directory
    // (e.g. "/home/fred/"). The path includes the trailing /, so
    // callers shouldn't mess with directory separators.
    //
    static QString homeDirPath();

    //
    // This user's identity on the system
    //
    static const char *getUserName();

    //
    // Preference booleans loaded from the QPE user settings file
    //
    static bool preferSystemIcons;
    static bool maximiseApplication;
    static bool maximiseDialogs;

    //
    // A value that is incremented each time this application starts
    // for this user
    //
    static int bootCount;
  };

#endif
