/*
 *  Copyright (C) 2003  Clive Butler
 *
 *  Author's email address: cebdev@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <classlib/String.hpp>

#include "BrowserSnmpProfile.hpp"

BrowserSnmpProfile::BrowserSnmpProfile()
          : name(0), timeout(50), retries(1), port(161), version(v1),
            readCommunity(0), writeCommunity(0),
            securityName(0),
            authProtocol(authHmacMd5), authPassword(0),
            privProtocol(privAes128), privPassword(0) {
  }

BrowserSnmpProfile::~BrowserSnmpProfile() {
  delete name;
  delete readCommunity;
  delete writeCommunity;
  delete securityName;
  delete authPassword;
  delete privPassword;
  }

BrowserSnmpProfile::BrowserSnmpProfile(const BrowserSnmpProfile& other)
              : name(0), timeout(other.timeout), retries(other.retries),
                port(other.port), version(other.version),
                readCommunity(0), writeCommunity(0),
                securityName(0),
                authProtocol(other.authProtocol), authPassword(0),
                privProtocol(other.privProtocol), privPassword(0) {

  name = new String(*other.name);

  if (other.readCommunity)
    readCommunity = new String(*other.readCommunity);

  if (other.writeCommunity)
    writeCommunity = new String(*other.writeCommunity);

  if (other.securityName)
    securityName = new String(*other.securityName);

  if (other.authPassword)
    authPassword = new String(*other.authPassword);

  if (other.privPassword)
    privPassword = new String(*other.privPassword);
  }

BrowserSnmpProfile& BrowserSnmpProfile::operator=(
            const BrowserSnmpProfile& other) {

  delete name;
  name = new String(*other.name);

  version = other.version;

  timeout = other.timeout;
  retries = other.retries;
  port = other.port;

  delete readCommunity;
  readCommunity = 0;
  if (other.readCommunity) readCommunity = new String(*other.readCommunity);

  delete writeCommunity;
  writeCommunity = 0;
  if (other.writeCommunity) writeCommunity = new String(*other.writeCommunity);

  authProtocol = other.authProtocol;
  delete authPassword;
  authPassword = 0;
  if (other.authPassword) authPassword = new String(*other.authPassword);

  privProtocol = other.privProtocol;
  delete privPassword;
  privPassword = 0;
  if (other.privPassword) privPassword = new String(*other.privPassword);

  return *this;
  }

void BrowserSnmpProfile::setName(const char *name) {
  delete this -> name;
  this -> name = new String(name);
  }

const char *BrowserSnmpProfile::getName() const {
  if (! name) return 0;
  return name -> getChars();
  }

void BrowserSnmpProfile::setTimeout(int timeout) {
  this -> timeout = timeout;
  }

int BrowserSnmpProfile::getTimeout() const {
  return timeout;
  }

void BrowserSnmpProfile::setRetries(int retries) {
  this -> retries = retries;
  }

int BrowserSnmpProfile::getRetries() const {
  return retries;
  }

void BrowserSnmpProfile::setPort(int port) {
  this -> port = port;
  }

int BrowserSnmpProfile::getPort() const {
  return port;
  }

void BrowserSnmpProfile::setVersion(SnmpVersion version) {
  this -> version = version;
  }

BrowserSnmpProfile::SnmpVersion BrowserSnmpProfile::getVersion() const {
  return version;
  }

void BrowserSnmpProfile::setReadCommunity(const char *community) {
  delete readCommunity;
  readCommunity = new String(community);
  }

const char *BrowserSnmpProfile::getReadCommunity() const {
  if (! readCommunity) return 0;
  return readCommunity -> getChars();
  }

void BrowserSnmpProfile::setWriteCommunity(const char *community) {
  delete writeCommunity;
  writeCommunity = new String(community);
  }

const char *BrowserSnmpProfile::getWriteCommunity() const {
  if (! writeCommunity) return 0;
  return writeCommunity -> getChars();
  }

void BrowserSnmpProfile::setSecurityName(const char *securityNameArg) {
  delete securityName;
  securityName = new String(securityNameArg);
  }

const char *BrowserSnmpProfile::getSecurityName() const {
  if (! securityName) return 0;
  return securityName -> getChars();
  }

void BrowserSnmpProfile::setAuthProtocol(SnmpAuthProtocol authProtocolArg) {
  authProtocol = authProtocolArg;
  }

BrowserSnmpProfile::SnmpAuthProtocol
            BrowserSnmpProfile::getAuthProtocol() const {
  return authProtocol;
  }

void BrowserSnmpProfile::setAuthPassword(const char *authPasswordArg) {
  delete authPassword;
  authPassword = new String(authPasswordArg);
  }

const char *BrowserSnmpProfile::getAuthPassword() const {
  if (! authPassword) return 0;
  return authPassword -> getChars();
  }

void BrowserSnmpProfile::setPrivProtocol(SnmpPrivProtocol privProtocolArg) {
  privProtocol = privProtocolArg;
  }

BrowserSnmpProfile::SnmpPrivProtocol
          BrowserSnmpProfile::getPrivProtocol() const {
  return privProtocol;
  }

void BrowserSnmpProfile::setPrivPassword(const char *privPasswordArg) {
  delete privPassword;
  privPassword = new String(privPasswordArg);
  }

const char *BrowserSnmpProfile::getPrivPassword() const {
  if (! privPassword) return 0;
  return privPassword -> getChars();
  }
