/*
 *  Copyright (C) 2003  Clive Butler
 *
 *  Author's email address: cebdev@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef BROWSERSNMPPROFILE_H
#define BROWSERSNMPPROFILE_H

#include <classlib/Object.hpp>

class String;

class BrowserSnmpProfile : public Object {
  public:
    enum SnmpVersion { v1, v2c, v3 };
    enum SnmpAuthProtocol { authNone, authHmacMd5, authHmacSha };
    enum SnmpPrivProtocol { privNone, privDes, privAes128, privAes192,
                            privAes256 };

    BrowserSnmpProfile();
    ~BrowserSnmpProfile();

    BrowserSnmpProfile(const BrowserSnmpProfile& other);
    BrowserSnmpProfile& operator=(const BrowserSnmpProfile& other);

    void setName(const char *name);
    const char *getName() const;

    void setTimeout(int timeout); // mS
    int getTimeout() const;

    void setRetries(int retries); // 0 means no *additional* tries
    int getRetries() const;

    void setPort(int port);
    int getPort() const;

    void setVersion(SnmpVersion version);
    SnmpVersion getVersion() const;

    void setReadCommunity(const char *community);
    const char *getReadCommunity() const;

    void setWriteCommunity(const char *community);
    const char *getWriteCommunity() const;

    void setSecurityName(const char *securityName);
    const char *getSecurityName() const;

    void setAuthProtocol(SnmpAuthProtocol authProtocol);
    SnmpAuthProtocol getAuthProtocol() const;

    void setAuthPassword(const char *authPassword);
    const char *getAuthPassword() const;

    void setPrivProtocol(SnmpPrivProtocol privProtocol);
    SnmpPrivProtocol getPrivProtocol() const;

    void setPrivPassword(const char *privPassword);
    const char *getPrivPassword() const;

    int getSecurityLevel() const;

  private:
    String *name;
    int timeout;
    int retries;
    int port;
    SnmpVersion version;

    // Community-based parameters
    String *readCommunity;
    String *writeCommunity;

    // USM parameters
    String *securityName;
    SnmpAuthProtocol authProtocol;
    String *authPassword;
    SnmpPrivProtocol privProtocol;
    String *privPassword;
  };

#endif
