/*
 *  Copyright (C) 2003  Clive Butler
 *
 *  Author's email address: cebdev@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <cstdio>

#include <qwidget.h>
#include <qfontmetrics.h>

#include "BrowserTextViewWidget.hpp"

//
// Above this value we don't report any sizeHint() changes
//
const int HeightThreshold = 1000;

BrowserTextViewWidget::BrowserTextViewWidget(QWidget *parent)
              : QTextView(parent), docHeight(0), currentLineHeight(0) {

  }

QSize BrowserTextViewWidget::sizeHint() const {
  //
  // Limit the size hint to a reasonable range
  //
  if (docHeight > HeightThreshold) return QSize(width(),HeightThreshold);
  else return QSize(width(),docHeight);
  }

QSize BrowserTextViewWidget::minimumSizeHint() const {
  return QSize(40,40);
  }

//
// We assume that only the appendString() method is used to add text.
// Note that each call could result in a sizeHint() change, so it's
// worthwhile to call with multiple lines at a time (hint, hint).
//
void BrowserTextViewWidget::appendString(const QString& string) {
  //
  // TBD - Do this in a fontChanged() slot
  //
  if (! currentLineHeight) {
    QFontMetrics metrics = fontMetrics();
    currentLineHeight = metrics.height();
    }

  //
  // Record the change in the document's height (we're going for an
  // approximation to yield a hint for layout managers in calculating
  // proportional sizes of children, so we don't worry about word-wrap)
  //
  const char *lp = string;
  while (lp) {
    lp = strchr(lp,'\n');
    if (lp) {
      lp++ ;

      docHeight += currentLineHeight;
      }
    }

  //
  // Ask the base class to do the real work
  //
  QTextView::append(string);

  //
  // If the document height is still below the threshold, report
  // a sizeHint() change
  //
  if (docHeight <= HeightThreshold) updateGeometry();
  }
