/*
 *  Copyright (C) 2003  Clive Butler
 *
 *  Author's email address: cebdev@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <qtable.h>
#include <qlayout.h>
#include <qpushbutton.h>

#include "FunctionParameterFormImpl.hpp"

#include "BrowserReportFunctionParameter.hpp"

FunctionParameterFormImpl::FunctionParameterFormImpl(QWidget* parent)
    : FunctionParameterForm(parent,0,true,0),
      defaultObjectIdentifier(0), defaultTableName(0) {

  ParameterTable -> setLeftMargin(0);
  ParameterTable -> verticalHeader() -> hide();
  ParameterTable -> horizontalHeader() -> setLabel(0,tr("Name"));
  ParameterTable -> horizontalHeader() -> setLabel(1,tr("Value"));
  ParameterTable -> setColumnStretchable(0,false);
  ParameterTable -> setColumnStretchable(1,true);

#ifndef QWS
  QFrame *buttonFrame = new QFrame(this);

  QHBoxLayout *buttonFrameLayout = new QHBoxLayout(buttonFrame);

  QPushButton *okButton = new QPushButton(buttonFrame);
  okButton -> setText("Ok");
  okButton -> adjustSize();
  okButton -> setDefault(true);

  QPushButton *cancelButton = new QPushButton(buttonFrame);
  cancelButton -> setText("Cancel");
  cancelButton -> adjustSize();

  buttonFrameLayout -> addStretch(1);
  buttonFrameLayout -> addWidget(okButton);
  buttonFrameLayout -> addWidget(cancelButton);

  layout() -> add(buttonFrame);

  connect(okButton,SIGNAL(clicked()),this,SLOT(accept()));
  connect(cancelButton,SIGNAL(clicked()),this,SLOT(reject()));
#endif
  }

FunctionParameterFormImpl::~FunctionParameterFormImpl() {
  // no need to delete child widgets, Qt does it all for us
  }

void FunctionParameterFormImpl::setDefaultObjectIdentifier(
                                              const char *oidArg) {
  defaultObjectIdentifier = oidArg;
  }

void FunctionParameterFormImpl::setDefaultTableName(const char *tableNameArg) {
  defaultTableName = tableNameArg;
  }

void FunctionParameterFormImpl::addParameter(
                                   BrowserReportFunctionParameter *parameter) {

  int currentRows = ParameterTable -> numRows();
  ParameterTable -> setNumRows(++currentRows);
  ParameterTable -> setCurrentCell(0,1);

  QTableItem *promptItem = new QTableItem(ParameterTable,QTableItem::Never,
                                        parameter -> getPrompt());
  ParameterTable -> setItem(currentRows - 1,0,promptItem);

  const char *value = parameter -> getDefaultValue();
  if (! value) {
    if (defaultObjectIdentifier &&
        (parameter -> getType() ==
             BrowserReportFunctionParameter::ObjectIdentifierParameterType)) {
      value = defaultObjectIdentifier;
      }
    else if (defaultTableName &&
             (parameter -> getType() ==
                     BrowserReportFunctionParameter::TableNameParameterType)) {
      value = defaultTableName;
      }
    }
  QTableItem *valueItem = new QTableItem(ParameterTable,QTableItem::Always,
                                         value);
  ParameterTable -> setItem(currentRows - 1,1,valueItem);
  ParameterTable -> adjustColumn(0);
  ParameterTable -> adjustColumn(1);
  }

const char *FunctionParameterFormImpl::getParameterValue(int r) const {
  return ParameterTable -> text(r,1);
  }
