/*
 *  Copyright (C) 2003  Clive Butler
 *
 *  Author's email address: cebdev@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <cstdio>
#include <climits>

#include <qlayout.h>
#include <qframe.h>
#include <qlistview.h>
#include <qpainter.h>

#include "GraphFormImpl.hpp"

#include "StripChartWidget.hpp"

const int KeyColumn          = 0;
const int NameColumn         = 1;
const int CurrentValueColumn = 2;
const int LowValueColumn     = 3;
const int HighValueColumn    = 4;

class GraphFormListViewItem : public QListViewItem {
  public:
    GraphFormListViewItem(QListView *parent,const char *nameArg,
                          int traceIdArg,QColor colorArg)
                   : QListViewItem(parent),
                     traceId(traceIdArg), color(colorArg) {

      setText(NameColumn,nameArg);
      low = INT_MAX;
      high = INT_MIN;
      }

    ~GraphFormListViewItem() {}

    void paintCell(QPainter *p,const QColorGroup& cg,int column,
                   int width,int align) {
      if (column == KeyColumn) {
        int h = height();
        p -> fillRect(0,0,width,h,QBrush(cg.base()));

        p -> fillRect(0,2,width,h - 4,QBrush(QColor(0,0,0)));

        int y = (h - 4)/ 2 - 1;
        p -> fillRect(5,y + 2,width - 10,2,QBrush(color));
        }
      else QListViewItem::paintCell(p,cg,column,width,align);
      }

    int traceId;
    QColor color;
    int low;
    int high;
  };

GraphFormImpl::GraphFormImpl(QWidget* parent) : GraphForm(parent) {
  TraceList -> setColumnAlignment(CurrentValueColumn,AlignRight);
  TraceList -> setColumnAlignment(LowValueColumn,AlignRight);
  TraceList -> setColumnAlignment(HighValueColumn,AlignRight);

  chartLayout = new QVBoxLayout(ChartFrame);
  stripChart = new StripChartWidget(ChartFrame);
  stripChart -> setBackgroundColor(QColor(0,0,0));
  chartLayout -> addWidget(stripChart);
  ((QVBoxLayout *)layout()) -> setStretchFactor(ChartFrame,1);
  }

GraphFormImpl::~GraphFormImpl() {
  // no need to delete child widgets, Qt does it all for us
  }

int GraphFormImpl::addTrace(const char *name) {
  int traceId = stripChart -> addTrace();

  new GraphFormListViewItem(TraceList,name,traceId,
                            stripChart -> getColor(traceId));

  return traceId;
  }

void GraphFormImpl::setValue(int traceId,int value) {

  stripChart -> setValue(traceId,value);

  GraphFormListViewItem *item =
              (GraphFormListViewItem *)TraceList -> firstChild();
  while (item) {
    if (item -> traceId == traceId) {
      item -> setText(CurrentValueColumn,QString().setNum(value));

      if (value < item -> low) {
        item -> low = value;
        item -> setText(LowValueColumn,QString().setNum(value));
        }

      if (value > item -> high) {
        item -> high = value;
        item -> setText(HighValueColumn,QString().setNum(value));
        }

      break;
      }

    item = (GraphFormListViewItem *)item -> nextSibling();
    }
  }

void GraphFormImpl::stop() {
  stripChart -> setShiftPeriod(0);
  }

void GraphFormImpl::setShiftPeriod(int shiftPeriodArg) {
  stripChart -> setShiftPeriod(shiftPeriodArg);
  }

QSize GraphFormImpl::preferredSize() const {
  return QSize(20,40);
  }

