/*
 *  Copyright (C) 2003  Clive Butler
 *
 *  Author's email address: cebdev@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef MAINFORMIMPL_H
#define MAINFORMIMPL_H

#include <qfont.h>

#include "MainForm.hpp"

class QApplication;
class QTimer;
class BrowserSnmpTarget;
class BrowserSnmpSession;
class BrowserConfiguration;
class SearchMIBFormImpl;
class Smi;
class SmiOidTreeNode;
class ObjectList;
class BrowserMIBTreeItem;
class BrowserReportMenuTree;

class MainFormImpl : public MainForm { 
  Q_OBJECT

  friend class BrowserConfigurationManager;
  friend class BrowserReportManager;

  public:
    MainFormImpl(QApplication *application);
    ~MainFormImpl();

    void readConfiguration();
    void writeConfiguration();

  public slots:
    void executeSelectedOperationSlot();
    void checkOperationSlot();
    void cancelCurrentOperationSlot();
    void openConfigFormSlot();
    void newSlot();
    void infoSlot();
    void searchSlot();
    void editSlot();
    void deleteSlot();
    void helpSlot();
    void tabSwitchedSlot();
    void nodeSelectionChangedSlot();
    void reportSelectionChangedSlot();
    void mibTreeSelectionChangedSlot();

  private:
    QFont systemDefaultFont;

    BrowserConfiguration *configuration;
    ObjectList *reports;
    BrowserReportMenuTree *reportMenuTree;

    QApplication *application;
    Smi *smi;
    BrowserSnmpSession *session;
    BrowserSnmpTarget *target;
    ObjectList *operationList;
    QTimer *operationTimer;

    SearchMIBFormImpl *searchMIBForm;

    //
    // Control of the New, Edit and Delete toolbar icons on the main form.
    // These definitions are dependent on the MainForm.ui so be careful!
    //
    enum TabId { TabIdMIB=0, TabIdNodes=1, TabIdReports=2, TabIdResults=3};
    enum IconId { IconIdNew=0, IconIdInfo=1, IconIdEdit=2, IconIdDelete=3,
                  IconIdSearch=4 };
    bool iconStates[4][5]; // [TabId][IconId]
    void setIconStates();

    void switchToResultsPage();
    void finishedOperation(const char *statusText);
    void setStatusText(const char *text);
    void clearStatusText();
    void openNewNodeForm();
    void openEditSelectedNodeForm();
    void deleteSelectedNode();
    void openInfoOnSelectedObjectForm();
    bool matchMIBObject(const SmiOidTreeNode *node) const;
    BrowserMIBTreeItem *findNextMIBMatchChild(BrowserMIBTreeItem *node) const;
    BrowserMIBTreeItem *findNextMIBMatch(BrowserMIBTreeItem *node) const;
    void openSearchMIBForm();
    void checkExecuteState();
    void updateFonts();
    void rebuildNodeList();
    void rebuildMIBTree();
    void rebuildMIBTree(const SmiOidTreeNode *node);
    void rebuildMIBTree(BrowserMIBTreeItem *item,
                        const SmiOidTreeNode *node);
  };

#endif // MAINFORMIMPL_H
