/*
 *  Copyright (C) 2003  Clive Butler
 *
 *  Author's email address: cebdev@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef SNMPOPERATION_H
#define SNMPOPERATION_H

#include <classlib/ExceptionClass.hpp>

#include "BrowserConfiguration.hpp"
#include "BrowserSnmpProfile.hpp"
#include "BrowserSnmpSession.hpp"
#include "BrowserSnmpTarget.hpp"
#include "BrowserReportFunction.hpp"

class QWidget;
class ObjectList;
class Smi;

class SnmpOperation : public ExceptionClass {
  public:
    SnmpOperation();
    virtual ~SnmpOperation();

    enum State { NotInitialised, Running, Finished, Cancelled, Error };

    State getState() const;

    virtual void initialise() = 0; // Abstract base class;
    virtual void doWork() = 0;
    virtual void cancel();
    virtual int getSleepTime() const = 0; // ms til next doWork() call

    void setEnvironment(BrowserConfiguration *configuration,
                        const Smi *smi,
                        const BrowserSnmpSession *session,
                        const BrowserSnmpTarget *target,
                        const BrowserReportFunction *function,
                        ObjectList *parameterValues,
                        int options=0);

    virtual QWidget *createWidget(QWidget *parent) = 0;
    QWidget *getWidget() const;

  protected:
    const BrowserConfiguration *configuration;
    const Smi *smi;
    const BrowserSnmpSession *session;
    const BrowserSnmpTarget *target;
    const BrowserReportFunction *function;
    ObjectList *parameterValues;
    int options;
    State state;
    QWidget *widget;

  private:
    SnmpOperation(const SnmpOperation& other); // No copies
    const SnmpOperation& operator=(const SnmpOperation& other); // No copies
  };

#endif
