/*
 *  Copyright (C) 2003  Clive Butler
 *
 *  Author's email address: cebdev@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef EXCEPTIONCLASS_H
#define EXCEPTIONCLASS_H

#include "Object.hpp"

class LocalException {
  public:
    LocalException() { }
    virtual ~LocalException() { }

    virtual const char *getMessage() const = 0;
    virtual const char *getDiagnostic() const { return 0; };
  };

#define THROW(newException,retval) \
  { exception = newException ; return retval; }

#define RETHROW(thrownException,retVal) \
  { exception = thrownException ; thrownException = 0 ; return retVal; }

class ExceptionClass : public Object {
  public:
    virtual ~ExceptionClass() {
      delete exception;
      }

    virtual ExceptionClass& operator=(const ExceptionClass& other) {
      exception = 0;
      return *this;
      }

    virtual Object *copy() const {
      return new ExceptionClass(*this);
      }

    const LocalException *getException() const {
      return exception;
      }

    LocalException *takeException() {
      LocalException *returnedException = exception;
      exception = 0;
      return returnedException;
      }

    mutable LocalException *exception;

  protected:
    ExceptionClass() : exception(0) {}
    ExceptionClass(const ExceptionClass& other) : exception(0) {}
  };

#endif
