/*
 *  Copyright (C) 2003  Clive Butler
 *
 *  Author's email address: cebdev@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <cstdio>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include "String.hpp"
#include "DetailedException.hpp"

#include "LineNumberReader.hpp"

LineNumberReader::LineNumberReader(const char *fileNameArg)
         : fileName(0), lineNumber(1), fd(-1) {

  fd = ::open(fileNameArg,O_RDONLY);
  if (fd == -1) {
    THROW(new DetailedException(fileNameArg,strerror(errno)),);
    }

  fileName = new String(fileNameArg);
  }

LineNumberReader::~LineNumberReader() {
  if (fd != -1) ::close(fd);
  delete fileName;
  }

const char *LineNumberReader::getFileName() const {
  if (fileName) return fileName -> getChars();
  return 0;
  }

int LineNumberReader::getLineNumber() const {
  return lineNumber;
  }

void LineNumberReader::mark(int readahead) {
  markedPosition = lseek(fd,0,SEEK_CUR);
  markedLineNumber = lineNumber;
  }

void LineNumberReader::reset() {
  lseek(fd,markedPosition,SEEK_SET);
  lineNumber = markedLineNumber;
  }

int LineNumberReader::read() {
  char ch;

  switch (::read(fd,&ch,1)) {
    case -1:
      THROW(new DetailedException(strerror(errno)),0);
    case 0:
      return -1;
    default:
      if (ch == '\n') lineNumber++ ;
      return ch;
    }
  }
