/*
 *  Copyright (C) 2003  Clive Butler
 *
 *  Author's email address: cebdev@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef LIST_H
#define LIST_H

class ListEntry {
  public:
    ListEntry(const void *item);
    ~ListEntry();

  private:
    ListEntry(const ListEntry& other);
    ListEntry& operator=(const ListEntry& other);

    const void *item;
    ListEntry *next;

  friend class ListIterator;
  friend class List;
  };

class List;

class ListIterator {
  public:
    ListIterator();
    ListIterator(List *list);
    ListIterator(const ListIterator& other);
    ~ListIterator();

    ListIterator& operator=(const ListIterator& other);

    bool hasNext() const;
    const void *next();
    void remove();
    void insert(const void *item);

  private:
    List *list;
    ListEntry *previous;
    ListEntry *entry;
  };

class List {
  private:
    List(const List &other);
    List& operator=(const List& other);

    ListEntry *first;
    ListEntry *last;
    int entryCount;

  public:
    List();
    virtual ~List();

    int count() const;
    virtual void add(const void *item);

    virtual ListIterator iterator();

  friend class ListIterator;
  };

#endif
