/*
 *  Copyright (C) 2003  Clive Butler
 *
 *  Author's email address: cebdev@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef OBJECTHASHMAP_H
#define OBJECTHASHMAP_H

#include "Object.hpp"

struct ObjectMapping {
  const Object *name;
  const Object *value;
  };

class ObjectChain;
class ObjectHashMap;

class ObjectHashMapIterator {
  public:
    ObjectMapping next();
    bool hasNext();
    void detach();

  friend class ObjectHashMap;

  private:
    ObjectHashMap *hashMap;
    int chainIndex;
    ObjectChain *previous;
    ObjectChain *current;
  };

class ObjectHashMap {
  public:
    ObjectHashMap();
    virtual ~ObjectHashMap();

    void add(Object *name,Object *value); // Map owns added objects
    void add(const char *name,const char *value); // Create Strings

    int size();
    ObjectHashMapIterator iterator();

    ObjectMapping find(const char *name) const;
    ObjectMapping get(const Object *name) const;
    void replaceValue(ObjectHashMapIterator it,const Object *newValue);
    char *getStats() const;                 // Caller must delete return value
    void remove(const char *name);
    void invalidate();

  friend class ObjectHashMapIterator;

  private:
    ObjectHashMap(const ObjectHashMap& other);
    ObjectHashMap& operator=(const ObjectHashMap& other);

    int mappingCount;
    ObjectChain **table;
  };

#endif
