/*
 *  Copyright (C) 2003  Clive Butler
 *
 *  Author's email address: cebdev@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <cstdlib>
#include <cstring>
#include <sys/param.h>

#include "StringBuffer.hpp"

static const int DEFAULT_SIZE_INCREMENT = 512;

StringBuffer::StringBuffer() : sizeIncrement(DEFAULT_SIZE_INCREMENT),
                               buffer(0), bufferCurrentPtr(0),
                               bufferCurrentAllocSize(0),
                               bufferFreeSpace(0) {

  buffer = new char[sizeIncrement];
  bufferCurrentAllocSize = sizeIncrement;
  bufferCurrentPtr = buffer;
  bufferFreeSpace = bufferCurrentAllocSize;
  *buffer = '\0';
  }

StringBuffer::StringBuffer(int sizeIncrement)
                 : sizeIncrement(sizeIncrement), buffer(0),
                   bufferCurrentPtr(0), bufferCurrentAllocSize(0),
                   bufferFreeSpace(0) {

  buffer = new char[sizeIncrement];
  bufferCurrentAllocSize = sizeIncrement;
  bufferCurrentPtr = buffer;
  bufferFreeSpace = bufferCurrentAllocSize;
  *buffer = '\0';
  }

StringBuffer::~StringBuffer() {
  delete[] buffer;
  }

void StringBuffer::append(const char *string,int length) {
  if (length >= bufferFreeSpace) {
    //
    // The string won't fit in the remaining free space, so we
    // need to re-allocate the buffer to be at least large enough
    // for the new contents, and for the increase to be no smaller
    // than the size increment
    //
    int increase = MAX(length + 1,sizeIncrement);
    int bufferCurrentSize = bufferCurrentPtr - buffer;
    bufferCurrentAllocSize += increase;
    bufferFreeSpace += increase;
    char *newBuffer = new char[bufferCurrentAllocSize];
    std::memcpy(newBuffer,buffer,bufferCurrentSize);
    delete[] buffer;
    buffer = newBuffer;
    bufferCurrentPtr = buffer + bufferCurrentSize;
    }

  if (length) {
    std::memcpy(bufferCurrentPtr,string,length);
    bufferCurrentPtr += length;
    *bufferCurrentPtr = '\0';
    bufferFreeSpace -= length;
    }
  }

void StringBuffer::append(const char *string) {
  append(string,std::strlen(string));
  }

void StringBuffer::append(char character) {
  append(&character,1);
  }

const char *StringBuffer::getChars() const {
  return buffer;
  }

//
// Note that ownership of the string is *transferred* to the caller,
// so he has to free it with StringBuffer::free()
//
char *StringBuffer::detach() {
  char *string = buffer;
  buffer = 0;

  if (! string) {
    string = new char[1];

    *string = '\0';
    }

  return string;
  }

void StringBuffer::free(char *string) {
  delete[] string;
  }
