/*
 *  Copyright (C) 2003  Clive Butler
 *
 *  Author's email address: cebdev@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef XMLH
#define XMLH

#include <unistd.h>

#include "DetailedException.hpp"

typedef void *XmlElement;

class Xml : public ExceptionClass {
  public:
    Xml();
    Xml(const char *xmlPath);

    ~Xml();

    void loadXmlFromString(const char *xmlString);
    void loadXmlFromBuffer(const char *buffer,int length);
    void loadXmlFromFile(const char *xmlPath);

    XmlElement getRootElement() const;
    XmlElement setRootElement(const char *name);

    XmlElement addElement(XmlElement parent,const char *elementName);
    void deleteElement(XmlElement element);

    const char *getElementName(XmlElement element) const;
    int getElementChildCount(XmlElement parent) const;
    XmlElement getElementChild(XmlElement parent,int i) const;
    const char *getElementContents(XmlElement element) const;
    void setElementContents(XmlElement element,const char *contents);

    void addElementAttribute(XmlElement element,const char *name,
                             const char *value);
    int getElementAttributeCount(XmlElement element) const;
    const char *getElementAttributeName(XmlElement element,int i) const;
    const char *getElementAttributeValue(XmlElement element,int i) const;

    char *xmlString() const;                      // Xml::free() after use
    char *xmlString(XmlElement element) const;    // Xml::free() after use
    static void free(char *string);

  private:
    Xml(const Xml& other);
    Xml& operator=(const Xml& other);

    void *tree;
  };

#endif
