/*
 *  Copyright (C) 2003  Clive Butler
 *
 *  Author's email address: cebdev@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <cstdio>
#include <cstring>
#include <cctype>
#include <cstdlib>

#include <classlib/List.hpp>

#include "FileLoadOperation.hpp"
#include "ModuleManager.hpp"

#include "SmiLoadOperation.hpp"
#include "XmlLoadOperation.hpp"

class FileLoadOperationPrivate {
  public:
    FileLoadOperationPrivate() : loader(0) {
      }
    ~FileLoadOperationPrivate() {
      delete loader;
      }

    XmlDbLoadOperation *loader;
    ListIterator pathListIterator;
  };

FileLoadOperation::FileLoadOperation(ModuleManager *manager)
          : XmlDbLoadOperation(manager), data(0) {
  data = new FileLoadOperationPrivate();
  }

FileLoadOperation::~FileLoadOperation() {
  delete data;
  }

void FileLoadOperation::initialise(const char *path) {
  XmlDbOperation::initialise();

  int length = std::strlen(path);
  if ((length >= 4) && ((std::strcmp(path + length - 4,".xml") == 0) ||
                        (std::strcmp(path + length - 4,".XML") == 0))) {
    data -> loader = new XmlLoadOperation(manager);
    }
  else {
    data -> loader = new SmiLoadOperation(manager);
    }

  data -> loader -> initialise(path);
  }

void FileLoadOperation::initialise(List *pathList) {
  XmlDbOperation::initialise();

  data -> pathListIterator = pathList -> iterator();
  if (data -> pathListIterator.hasNext()) {
    const char *path = (const char *)data -> pathListIterator.next();
    initialise(path);
    }
  }

void FileLoadOperation::doWork() {
  switch (data -> loader -> getState()) {
    case XmlDbOperation::Running:
      data -> loader -> doWork();
      break;
    case XmlDbOperation::Completed:
      if (data -> pathListIterator.hasNext()) {
        const char *path = (const char *)data -> pathListIterator.next();
        initialise(path);
        }
      else state = XmlDbOperation::Completed;
      break;
    default:
      state = data -> loader -> getState();
      RETHROW(data -> loader -> exception,);
    }
  }

const char *FileLoadOperation::getCurrentModuleName() const {
  return data -> loader -> getCurrentModuleName();
  }

int FileLoadOperation::getProgress() const {
  return data -> loader -> getProgress();
  }
