/*
 *  Copyright (C) 2003  Clive Butler
 *
 *  Author's email address: cebdev@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <cstdio>
#include <cstdlib>
#include <cerrno>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <classlib/Xml.hpp>

#ifdef QWS
#include <qpe/config.h>
#else
#include <myqpe/Config.hpp>
#endif

#include <qmessagebox.h>

#include "ManagerConfigurationManager.hpp"
#include "ManagerFormImpl.hpp"
#include "ManagerResource.hpp"

ManagerConfigurationManager::ManagerConfigurationManager(
                                    ManagerFormImpl *parent)
               : parent(parent) {
  }

ManagerConfigurationManager::~ManagerConfigurationManager() {
  }

static void loadPreferences(ManagerConfiguration *configuration) {
  Config preferences(ManagerResource::applicationName(),Config::User);
  preferences.setGroup("General");

  configuration -> preferSystemIcons =
                       preferences.readBoolEntry("preferSystemIcons",false);

  QString applicationFontFamily =
                 preferences.readEntry("applicationFontFamily");
  if (applicationFontFamily.isNull())
    configuration -> applicationFontFamily = 0;
  else {
    configuration -> applicationFontFamily = new String(applicationFontFamily);
    }

  QString applicationFontSize = preferences.readEntry("applicationFontSize");
  if (applicationFontSize.isNull())
    configuration -> applicationFontSize = 0;
  else
    configuration -> applicationFontSize = applicationFontSize.toInt();
  }

static void savePreferences(ManagerConfiguration *configuration) {
  Config preferences(ManagerResource::applicationName(),Config::User);
  preferences.setGroup("General");

  preferences.writeEntry("preferSystemIcons",
                         configuration -> preferSystemIcons);

  if (configuration -> applicationFontFamily) {
    preferences.writeEntry("applicationFontFamily",
                           configuration -> applicationFontFamily ->
                                                     getChars());
    }
  else preferences.removeEntry("applicationFontFamily");

  if (configuration -> applicationFontSize != 0) {
    preferences.writeEntry("applicationFontSize",
                           QString().setNum(configuration ->
                                                  applicationFontSize));
    }
  else preferences.removeEntry("applicationFontSize");
  }

void ManagerConfigurationManager::load() {

  parent -> configuration = new ManagerConfiguration();
  loadPreferences(parent -> configuration);
  }

void ManagerConfigurationManager::save() {

  savePreferences(parent -> configuration);
  }
