/*
 *  Copyright (C) 2003  Clive Butler
 *
 *  Author's email address: cebdev@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <cstdio>
#include <cstdlib>
#include <cstring>

#ifdef QWS
#include <qpe/qpeapplication.h>
#include <qpe/config.h>
#include <qpe/global.h>
#else
#include <myqpe/Config.hpp>
#include <myqpe/Resource.hpp>
#endif

#include "ManagerResource.hpp"

#ifdef QWS
bool ManagerResource::preferSystemIcons = false;
bool ManagerResource::maximiseApplication = true;
bool ManagerResource::maximiseDialogs = true;
#else
bool ManagerResource::preferSystemIcons = true;
bool ManagerResource::maximiseApplication = false;
bool ManagerResource::maximiseDialogs = false;
#endif

void ManagerResource::loadPreferences() {
  Config preferences(applicationName(),Config::User);
  preferences.setGroup("General");
  preferSystemIcons = preferences.readBoolEntry("preferSystemIcons",
                                                preferSystemIcons);

  maximiseApplication = preferences.readBoolEntry("maximiseApplication",
                                                  maximiseApplication);

  maximiseDialogs = preferences.readBoolEntry("maximiseDialogs",
                                              maximiseDialogs);

  preferences.writeEntry("configured",true);
  }

const char *ManagerResource::applicationName() {
  return "SNMPzMIBManager";
  }

const char *ManagerResource::applicationGroupName() {
  return "SNMPz";
  }

const char *ManagerResource::applicationTitle() {
  return "SNMP MIB Manager";
  }

const char *ManagerResource::configurationFileName() {
  return "Configuration.xml";
  }

const char *ManagerResource::databaseFileName() {
  return "smi.db";
  }

QPixmap ManagerResource::loadPixmap(const char *name) {
#ifdef QWS
  //
  // Qtopia has a built-in path (not to mention a built-in set of icons)
  // so the names we give it are always relative
  //
  QString systemPixmapPath = name;

  QString applicationPixmapPath = applicationGroupName();
  applicationPixmapPath = applicationPixmapPath + "/" + name;
#else
  QString systemPixmapPath = "/usr/share/icons/crystalsvg/16x16/actions/";
  systemPixmapPath = systemPixmapPath + name;

  QString applicationPixmapPath = "/opt/";
  applicationPixmapPath = applicationPixmapPath + applicationGroupName() +
                          "/icons/" + name;
#endif

  QPixmap pixmap;
  if (preferSystemIcons) {
    pixmap = Resource::loadPixmap(systemPixmapPath);
    if (! pixmap.isNull()) return pixmap;

    pixmap = Resource::loadPixmap(applicationPixmapPath);
    if (! pixmap.isNull()) return pixmap;
    }
  else {
    pixmap = Resource::loadPixmap(applicationPixmapPath);
    if (! pixmap.isNull()) return pixmap;

    pixmap = Resource::loadPixmap(systemPixmapPath);
    if (! pixmap.isNull()) return pixmap;
    }

  return pixmap;
  }

QString ManagerResource::homeDirPath() {
  QString path = getenv("HOME");
  path += "/";

  return path;
  }

QString ManagerResource::getApplicationDirUser() {
  QString path = homeDirPath();
#ifdef QWS
  path = path + "Applications/" + applicationName() + "/";
#else
  path = path + "." + applicationName() + "/";
#endif
  return path;
  }

QString ManagerResource::getApplicationGroupDirUser() {
  QString path = homeDirPath();
#ifdef QWS
  path = path + "Applications/" + applicationGroupName() + "/";
#else
  path = path + "." + applicationGroupName() + "/";
#endif
  return path;
  }

QString ManagerResource::getApplicationSharedDir() {
#ifdef QWS
  QString path = QPEApplication::qpeDir() + "etc/" +
                          applicationName() + "/";
#else
  QString path = getApplicationGroupSharedDir();
  path = path + applicationName() + "/";
#endif
  return path;
  }

QString ManagerResource::getApplicationGroupSharedDir() {
#ifdef QWS
  return QPEApplication::qpeDir() + "etc/" + applicationGroupName() + "/";
#else
  QString path = "/opt/";
  path = path + applicationGroupName() + "/";
  return path;
#endif
  }

QString ManagerResource::expandVariables(const char *path) {
  QString result;
  while (*path) {
    if ((*path == '$') && (*(path + 1) == '{')) {
      path += 2;
      const char *endPtr = std::strchr(path,'}');
      if (! endPtr) return result;
      int length = endPtr - path;
      if ((length == 4) && (strncmp(path,"HOME",length) == 0)) {
        result += homeDirPath();
        }
      else if ((length == 14) &&
               (strncmp(path,"APPGROUPSHARED",length) == 0)) {
        result += getApplicationGroupSharedDir();
        }
      else if ((length == 2) && (strncmp(path,"SD",length) == 0)) {
        result += "/mnt/card";
        }
      else if ((length == 2) && (strncmp(path,"CF",length) == 0)) {
        result += "/mnt/cf";
        }
      else if ((length == 9) && (strncmp(path,"APPSHARED",length) == 0)) {
        result += getApplicationSharedDir();
        }
      else if ((length == 7) && (strncmp(path,"APPUSER",length) == 0)) {
        result += getApplicationDirUser();
        }
      else if ((length == 12) &&
               (strncmp(path,"APPGROUPUSER",length) == 0)) {
        result += getApplicationGroupDirUser();
        }
      // else null

      path += length + 1;
      }
    else result += *path;
    }

  return result;
  }
