/*
 *  Copyright (C) 2003  Clive Butler
 *
 *  Author's email address: cebdev@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <cstdio>
#include <cstring>

#include "ModuleLoadSelectionDialogImpl.hpp"
#include "ManagerResource.hpp"

#include <qcombobox.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qlistview.h>
#include <qdir.h>

//
// List of locations from which MIB files can be loaded
//
struct { char *name; char *path; } dirList[] = {
    { "Home",         "${HOME}" },
    { "SNMPz Shared", "${APPGROUPSHARED}" },
    { "SD Card",      "${SD}" },
    { "CF Card",      "${CF}" },
    { 0,              0 }
  };

ModuleLoadSelectionDialogImpl::ModuleLoadSelectionDialogImpl(
                                         QWidget* parent)
    : ModuleLoadSelectionDialog(parent,0,true,0),
      directory(0) {

#ifndef QWS
  QFrame *buttonFrame = new QFrame(this);

  QHBoxLayout *buttonFrameLayout = new QHBoxLayout(buttonFrame);

  QPushButton *okButton = new QPushButton(buttonFrame);
  okButton -> setText("Ok");
  okButton -> adjustSize();
  okButton -> setDefault(true);

  QPushButton *cancelButton = new QPushButton(buttonFrame);
  cancelButton -> setText("Cancel");
  cancelButton -> adjustSize();

  buttonFrameLayout -> addStretch(1);
  buttonFrameLayout -> addWidget(okButton);
  buttonFrameLayout -> addWidget(cancelButton);

  layout() -> add(buttonFrame);

  connect(okButton,SIGNAL(clicked()),this,SLOT(accept()));
  connect(cancelButton,SIGNAL(clicked()),this,SLOT(reject()));
#endif

  //
  // Add the list of locations to the dialog
  //
  int d = 0;
  while (dirList[d].name) {
    LocationComboBox -> insertItem(dirList[d].name);
    d++ ;
    }
  locationChangedSlot();
  }

ModuleLoadSelectionDialogImpl::~ModuleLoadSelectionDialogImpl() {
  // no need to delete child widgets, Qt does it all for us

  delete directory;
  }

void ModuleLoadSelectionDialogImpl::locationChangedSlot() {
  int d = 0;
  while (dirList[d].name) {
    if (strcmp(LocationComboBox -> currentText(),dirList[d].name) == 0) {
      loadDirectory(ManagerResource::expandVariables(dirList[d].path));
      }
    d++ ;
    }

//  if (strcmp(LocationComboBox -> currentText(),"Home") == 0) {
//    loadDirectory(ManagerResource::homeDirPath());
//    }
//  else if (strcmp(LocationComboBox -> currentText(),"Shared") == 0) {
//    loadDirectory(ManagerResource::getApplicationGroupSharedDir());
//    }
  }

void ModuleLoadSelectionDialogImpl::loadDirectory(const char *path) {
  AvailableModulesList -> clear();
  delete directory;

  directory = new QDir(path,QString::null,QDir::Name,QDir::Files);
  int fileCount = directory -> count();
  for (int i = 0 ; i < fileCount ; i++ ) {
    QListViewItem *item = new QListViewItem(AvailableModulesList);
    item -> setText(0,(*directory)[i]);
    }
  }

QStringList ModuleLoadSelectionDialogImpl::selectedFiles() const {
  QStringList pathList;
  QListViewItem *item = AvailableModulesList -> firstChild();
  while (item) {
    if (item -> isSelected()) {
      pathList.append(directory -> absFilePath(item -> text(0)));
      }

    item = item -> nextSibling();
    }

  return pathList;
  }

void ModuleLoadSelectionDialogImpl::reloadSlot() {
  locationChangedSlot();
  }
