/*
 *  Copyright (C) 2003  Clive Butler
 *
 *  Author's email address: cebdev@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef MODULEMANAGER_H
#define MODULEMANAGER_H

#include <classlib/ExceptionClass.hpp>
#include <classlib/Xml.hpp>
#include <classlib/ObjectHashMap.hpp>

class List;
class Db;
class XmlDbOperation;

class ModuleInfo;
class ModuleManagerPrivate;

class ModuleManager : public ExceptionClass {

  friend class XmlLoadOperation;
  friend class XmlUnloadOperation;
  friend class SmiLoadOperation;

  public:
    ModuleManager();
    ~ModuleManager();

    void openDb(const char *path);
    XmlDbOperation *dbLoader(const char *path);
    XmlDbOperation *dbLoader(List *pathList);
    XmlDbOperation *dbUnloader(const char *moduleName);
    void finishedEditing(bool ok);
    void deleteDatabase();

    List *getModuleList() const;
    Db *getDb() const;

  private:
    ModuleManager(const ModuleManager& other);
    ModuleManager& operator=(const ModuleManager& other);

    void writeXml(Xml& xml,const char *key);
    void writeXml(Xml& xml,XmlElement element,const char *key);
    void addModule(const char *moduleName);
    void deleteModule(const char *moduleName);
    void saveModuleList();
    void addOidMapping(const char *name,const char *value);
    void replaceOidValue(ModuleInfo *moduleInfo,
                                    ObjectHashMapIterator it,
                                    ObjectMapping mapping,
                                    const char *newValue);
    const char *resolveOid(const char *value);
    void resolveOids();
    void saveOidMap();

    ModuleManagerPrivate *data;
  };

#endif
