/*
 *  Copyright (C) 2003  Clive Butler
 *
 *  Author's email address: cebdev@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef SMILOADOPERATION_H
#define SMILOADOPERATION_H

#include "XmlDbLoadOperation.hpp"

#include <classlib/Xml.hpp>

#include "SmiToken.hpp"

class SmiLoadOperationPrivate;
class SmiOidValue;
class SmiValueReference;
class SmiIndexList;
class SmiSyntax;
class SmiComponentList;
class SmiRangeList;
class SmiSizeList;
class SmiEnumerationList;
class SmiDefaultValue;
class SmiIdentifierList;
class ObjectList;

class SmiLoadOperation : public XmlDbLoadOperation {
  public:
    SmiLoadOperation(ModuleManager *manager);
    ~SmiLoadOperation();

    void initialise(const char *path);
    void doWork();

    const char *getCurrentModuleName() const;
    int getProgress() const;

  private:
    SmiLoadOperation(const SmiLoadOperation& other);
    SmiLoadOperation& operator=(const SmiLoadOperation& other);

    SmiLoadOperationPrivate *data;

    SmiToken nextToken();
    SmiToken readWordToken();
    SmiToken readStringToken();
    void processAssignmentTail(Xml& xml,const char *identifier,
                               const char *assignmentTag);
    SmiDefaultValue *parseDefaultValue();
    SmiOidValue *parseOidValue();
    SmiValueReference *parseValueReference();
    SmiIndexList *parseIndexList();
    const char *buildOidString(SmiOidValue *oidValue);
    void createModule(const char *name);
    bool parseDefinition();
    void parseImports();
    void parseExports();
    SmiSyntax *parseSyntax();
    SmiSizeList *parseSizeList();
    SmiRangeList *parseRangeList();
    SmiComponentList *parseSyntaxList();
    SmiEnumerationList *parseEnumerationList();
    SmiIdentifierList *parseIdentifierList();
    ObjectList *parseValueReferenceList();
    void parseObjectType(const char *identifier);
    void parseObjectIdentifier(const char *identifier);
    void parseObjectIdentity(const char *identifier);
    void parseNotificationType(const char *identifier);
    void parseTrapType(const char *identifier);
    void parseModuleIdentity(const char *identifier);
    void parseComplianceGroup(Xml& xml,XmlElement groupsElement);
    void parseComplianceObject(Xml& xml,XmlElement objectsElement);
    void parseVariation(Xml& xml,XmlElement moduleElement);
    void parseModuleComplianceEntry(Xml& xml,XmlElement modulesElement);
    void parseModuleCompliance(const char *identifier);
    void parseAgentCapabilities(const char *identifier);
    void parseObjectGroup(const char *identifier);
    void parseNotificationGroup(const char *identifier);
    void parseMacro(const char *identifier);
    void parseTypedef(const char *identifier);
    void generateValueReferenceXml(Xml& xml,XmlElement parentElement,
                                   const char *tag,
                                   SmiValueReference *valueReference);
    void generateOidValueXml(Xml& xml,XmlElement parentElement,
                             SmiOidValue *oidValue);
    void generateSyntaxXml(Xml& xml,XmlElement parentElement,
                           const char *tag,SmiSyntax *syntax);
    void generateDefaultValueXml(Xml& xml,XmlElement parentElement,
                                 SmiDefaultValue *defaultValue);
    void writeModuleHeading();
    void writeModuleImports();
    void writeModuleIndexes();

  friend class SmiLoadOperationException;
  };

#endif
