/*
 *  Copyright (C) 2003  Clive Butler
 *
 *  Author's email address: cebdev@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <cstdlib>

#include "SmiToken.hpp"

#include <classlib/String.hpp>
#include <classlib/DetailedException.hpp>

SmiToken::SmiToken() : tokenType(TT_NULL), value(0) {
  }

SmiToken::SmiToken(Type tokenTypeArg,int sourceLineNumberArg)
      : tokenType(tokenTypeArg), sourceLineNumber(sourceLineNumberArg),
        value(0) {
  }

SmiToken::SmiToken(Type tokenTypeArg,int sourceLineNumberArg,
                   const char *valueArg)
      : tokenType(tokenTypeArg), sourceLineNumber(sourceLineNumberArg),
        value(0) {
  value = new String(valueArg);
  }

SmiToken::~SmiToken() {
  delete value;
  }

SmiToken::SmiToken(const SmiToken& other)
             : tokenType(other.tokenType),
               sourceLineNumber(other.sourceLineNumber), value(0) {
  if (other.value) value = new String(other.value);
  }

SmiToken& SmiToken::operator=(const SmiToken& other) {
  tokenType = other.tokenType;
  sourceLineNumber = other.sourceLineNumber;

  delete value;
  if (other.value) value = new String(other.value);
  else value = 0;

  return *this;
  }

SmiToken::Type SmiToken::getTokenType() const {
  return tokenType;
  }

const char *SmiToken::getTokenValue() const {
  if (value) return value -> getChars();
  return 0;
  }

int SmiToken::getSourceLineNumber() const {
  return sourceLineNumber;
  }

int SmiToken::getIntValue() const {
  switch (tokenType) {
    case TT_NUMBER:  return strtol(value -> getChars(),0,10);
    case TT_HSTRING: return strtol(value -> getChars(),0,16);
    case TT_BSTRING: return strtol(value -> getChars(),0,2);
    default: break;
    }

  THROW(new DetailedException("Not a number"),0);
  }

long long SmiToken::getBigIntegerValue() const {
  switch (tokenType) {
    case TT_NUMBER:  return strtoll(value -> getChars(),0,10);
    case TT_HSTRING: return strtoll(value -> getChars(),0,16);
    case TT_BSTRING: return strtoll(value -> getChars(),0,2);
    default: break;
    }

  THROW(new DetailedException("Not a number"),0);
  }

