/*
 *  Copyright (C) 2003  Clive Butler
 *
 *  Author's email address: cebdev@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef XMLDBOPERATION_H
#define XMLDBOPERATION_H

#include <classlib/ExceptionClass.hpp>
#include <classlib/Xml.hpp>

class ModuleManager;

class XmlDbOperationException : public LocalException {
  public:
    XmlDbOperationException(const char *message);

    ~XmlDbOperationException();

    const char *getMessage() const { return message; }

  private:
    char *message;
  };

class XmlDbOperation : public ExceptionClass {
  public:
    enum State { NotStarted, Running, Completed, Error };

    XmlDbOperation(ModuleManager *manager);
    virtual ~XmlDbOperation();

    void initialise();
    virtual void doWork() = 0;
    State getState() const { return state; }

    virtual int getProgress() const;
    virtual const char *getCurrentModuleName() const;

  private:
    XmlDbOperation(const XmlDbOperation& other);
    XmlDbOperation& operator=(const XmlDbOperation& other);

  protected:
    ModuleManager *manager;
    State state;
  };

#endif
