/*
 *  Copyright (C) 2003  Clive Butler
 *
 *  Author's email address: cebdev@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef XMLLOADOPERATION_H
#define XMLLOADOPERATION_H

#include "XmlDbLoadOperation.hpp"

#include <classlib/Xml.hpp>
#include <classlib/ObjectHashMap.hpp>

class XmlLoadOperation : public XmlDbLoadOperation {
  public:
    XmlLoadOperation(ModuleManager *manager);
    ~XmlLoadOperation();

    void initialise(const char *path);
    void doWork();

    const char *getCurrentModuleName() const;
    int getProgress() const;

  private:
    XmlLoadOperation(const XmlLoadOperation& other);
    XmlLoadOperation& operator=(const XmlLoadOperation& other);

    //
    // The following are state variables for tracking the progress
    // of the operation
    //
    Xml *xml;
    int moduleCount;
    XmlElement root;
    int currentModuleIndex;
    const char *currentModuleName;

    const char *resolveOid(const char *value);
    void resolveOids();
    const char *buildOidString(XmlElement oidElement,ObjectHashMap& importsMap);

    void writeModule(XmlElement moduleElement);

    void writeModuleHeading(XmlElement moduleElement);

    void processModuleAssignment(XmlElement assignmentElement,
                                 const char *assignmentName,
                                 ObjectHashMap& importsMap);

    void processModuleAssignments(XmlElement assignmentsElement,
                                  ObjectHashMap& importsMap);

    void processModuleImportsFrom(XmlElement importModuleElement,
                                  const char *importModuleName,
                                  ObjectHashMap& importsMap);

    void processModuleImports(XmlElement importsElement,
                              ObjectHashMap& importsMap);

    void processModuleTypedef(XmlElement typedefElement,
                              const char *typedefName);

    void processModuleTypedefs(XmlElement typedefsElement);

    void writeModuleImports(ObjectHashMap& importsMap);
  };

#endif
