/*
 *  Copyright (C) 2003  Clive Butler
 *
 *  Author's email address: cebdev@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <cstdio>
#include <cstring>

#include "XmlUnloadOperation.hpp"

#include "ModuleManager.hpp"

#include <classlib/Xml.hpp>
#include <classlib/List.hpp>
#include <smidb/SmiDb.hpp>

XmlUnloadOperation::XmlUnloadOperation(ModuleManager *manager)
          : XmlDbOperation(manager), moduleName(0) {
  }

XmlUnloadOperation::~XmlUnloadOperation() {
  }

void XmlUnloadOperation::initialise(const char *moduleName) {
  XmlDbOperation::initialise();

  this -> moduleName = moduleName;
  }

void XmlUnloadOperation::doWork() {
  //
  // Remove the module name from the module list and OID map
  //
  manager -> deleteModule(moduleName);
  manager -> saveModuleList();
  manager -> saveOidMap();

  //
  // Now we have to delete all the database records relating to this
  // module
  //
  manager -> getDb() -> deleteRecord(moduleName);

  //
  // And we're done
  //
  state = Completed;
  }

int XmlUnloadOperation::getProgress() const {
  return 100;
  }
