/*
 *  Copyright (C) 2003  Clive Butler
 *
 *  Author's email address: cebdev@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef SMI_H
#define SMI_H

#include <classlib/ExceptionClass.hpp>
#include <classlib/String.hpp>

#include "SmiDb.hpp"
#include "SmiOidTree.hpp"
#include "SmiModuleList.hpp"
#include "SmiOid.hpp"
#include "SmiSyntax.hpp"
#include "SmiTypedef.hpp"
#include "SmiReferenceCache.hpp"

class Smi : public ExceptionClass {
  public:
    Smi();
    Smi(const char *dbPath);
    ~Smi();

    enum FormatOptions { LongOidNames=1, ShortValues=2 }; // Bit values

    void open(const char *dbPath);
    SmiModuleList *getModuleList() const;
    SmiOidTree *getOidTree() const;

    //
    // Utility functions
    //
    SmiOid *getOid(const char *name) const;

    ObjectList *getIndexValues(const SmiOid& oidInstance,int options) const;

    const SmiAssignment *
               Smi::findAssignment(const SmiModule *referringModule,
                                   const char *assignmentName,
                                   const SmiModule **definingModule) const;

    const SmiTypedef *findTypedef(const SmiModule *referringModule,
                                  const SmiSyntax *syntax,
                                  const SmiModule **definingModule) const;

    const SmiTypedef *findHintedTypedef(const SmiModule *referringModule,
                                  const SmiSyntax *syntax) const;

    const SmiSyntax *getBaseSyntax(const SmiModule *referringModule,
                                   const SmiSyntax *syntax) const;

    void getTypeInfo(const SmiModule& module,const SmiSyntax& syntax,
                     const SmiTypedef **baseTypedef,
                     const SmiSyntax **baseSyntax,const char **hint,
                     const SmiEnumerationList **enumerations) const;

    String *stringValue(const SmiOid& oid,int options=0) const;

    String *signedIntegerString(const SmiModule *referringModule,
                                const SmiSyntax *syntax,long value) const;

    String *unsignedIntegerString(const SmiModule *referringModule,
                                  const SmiSyntax *syntax,
                                  unsigned long value) const;

    String *timeticksString(const SmiModule *referringModule,
                            const SmiSyntax *syntax,
                            unsigned long value) const;

    String *octetString(const SmiModule *referringModule,
                        const SmiSyntax *syntax,
                        const unsigned char *buffer,
                        int length,int options=0) const;

    String *bitString(const SmiModule *referringModule,
                      const SmiSyntax *syntax,
                      const unsigned char *buffer,
                      unsigned long length) const;

  private:
    Smi(const Smi& other);
    Smi& operator=(const Smi& other);

    SmiDb *smiDb;
    SmiModuleList *moduleList;
    SmiOidTree *oidTree;
    SmiReferenceCache *referenceCache;
  };

#endif
