/*
 *  Copyright (C) 2003  Clive Butler
 *
 *  Author's email address: cebdev@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <cstdio>
#include <cstring>

#include "SmiAssignmentDriver.hpp"
#include "SmiObjectIdentityLoader.hpp"
#include "SmiObjectTypeLoader.hpp"
#include "SmiNotificationTypeLoader.hpp"

SmiAssignmentDriver::SmiAssignmentDriver(const SmiDb *smiDb,
                                         const SmiModule *module,
                                         const char *name)
              : SmiDriver(smiDb), module(module), loader(0) {

  char recordName[128];
  std::sprintf(recordName,"/MODULE/%s/assignments/%s",module -> getName(),name);
  setKey(recordName);
  }

SmiAssignmentDriver::~SmiAssignmentDriver() {
  delete loader;
  }

SmiAssignment *SmiAssignmentDriver::takeAssignment() {
  if (! loader) return 0;

  return loader -> takeAssignment();
  }

void SmiAssignmentDriver::loaded(const Xml& xml) {
  XmlElement root = xml.getRootElement();
  const char *type = xml.getElementName(root);
  if (std::strcmp(type,"objecttype") == 0) {
    loader = new SmiObjectTypeLoader(module,xml);
    loader -> load();
    }
  else if (std::strcmp(type,"objectid") == 0) {
    loader = new SmiObjectIdentityLoader(module,xml);
    loader -> load();
    }
  else if (std::strcmp(type,"notification") == 0) {
    loader = new SmiNotificationTypeLoader(module,xml);
    loader -> load();
    }
  // else TBD
  }
