/*
 *  Copyright (C) 2003  Clive Butler
 *
 *  Author's email address: cebdev@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <cstdio>
#include <cstdlib>

#include "SmiDefaultValue.hpp"

SmiDefaultValue::SmiDefaultValue() {
  }

SmiDefaultValue::~SmiDefaultValue() {
  }

SmiIntegerDefaultValue::SmiIntegerDefaultValue() : value(0) {
  }

SmiIntegerDefaultValue::SmiIntegerDefaultValue(const char *integerString) {
  value = strtoll(integerString,0,10);
  }

SmiIntegerDefaultValue::~SmiIntegerDefaultValue() {
  }

SmiDefaultValue::Type SmiIntegerDefaultValue::getType() const {
  return SmiDefaultValue::INTEGER_VALUE;
  }

long long SmiIntegerDefaultValue::getValue() const {
  return value;
  }

String *SmiIntegerDefaultValue::toString() const {
  char scratch[64];
  std::sprintf(scratch,"%lld",value);
  return new String(scratch);
  }

SmiHexStringDefaultValue::SmiHexStringDefaultValue(const char *hexString) {
  this -> hexString = new String(hexString);
  }

SmiHexStringDefaultValue::~SmiHexStringDefaultValue() {
  delete hexString;
  }

SmiDefaultValue::Type SmiHexStringDefaultValue::getType() const {
  return SmiDefaultValue::HEX_STRING_VALUE;
  }

const char *SmiHexStringDefaultValue::getValue() const {
  return hexString -> getChars();
  }

String *SmiHexStringDefaultValue::toString() const {
  return new String(hexString);
  }

SmiStringDefaultValue::SmiStringDefaultValue(const char *string) {
  value = new String(string);
  }

SmiStringDefaultValue::~SmiStringDefaultValue() {
  delete value;
  }

SmiDefaultValue::Type SmiStringDefaultValue::getType() const {
  return SmiDefaultValue::STRING_VALUE;
  }

const char *SmiStringDefaultValue::getValue() const {
  return value -> getChars();
  }

String *SmiStringDefaultValue::toString() const {
  return new String(value);
  }

SmiIdentifierDefaultValue::SmiIdentifierDefaultValue(const char *string) {
  value = new String(string);
  }

SmiIdentifierDefaultValue::~SmiIdentifierDefaultValue() {
  delete value;
  }

SmiDefaultValue::Type SmiIdentifierDefaultValue::getType() const {
  return SmiDefaultValue::IDENTIFIER_VALUE;
  }

const char *SmiIdentifierDefaultValue::getValue() const {
  return value -> getChars();
  }

String *SmiIdentifierDefaultValue::toString() const {
  return new String(value);
  }

SmiIdentifierListDefaultValue::SmiIdentifierListDefaultValue(
                                                SmiIdentifierList *idlist) {
  this -> idlist = idlist;
  }

SmiIdentifierListDefaultValue::~SmiIdentifierListDefaultValue() {
  delete idlist;
  }

SmiDefaultValue::Type SmiIdentifierListDefaultValue::getType() const {
  return SmiDefaultValue::IDLIST_VALUE;
  }

SmiIdentifierList *SmiIdentifierListDefaultValue::getValue() const {
  return idlist;
  }

String *SmiIdentifierListDefaultValue::toString() const {
  StringBuffer buffer;
  buffer.append("{");

  ListIterator it = idlist -> iterator();
  while (it.hasNext()) {
    buffer.append((char *)it.next());
    if (it.hasNext()) buffer.append(", ");
    }

  buffer.append("}");
  return new String(buffer);
  }

SmiBinaryStringDefaultValue::SmiBinaryStringDefaultValue(
                const char *binaryString) {
  this -> binaryString = new String(binaryString);
  }

SmiBinaryStringDefaultValue::~SmiBinaryStringDefaultValue() {
  delete binaryString;
  }

SmiDefaultValue::Type SmiBinaryStringDefaultValue::getType() const {
  return SmiDefaultValue::BINARY_STRING_VALUE;
  }

const char *SmiBinaryStringDefaultValue::getValue() const {
  return binaryString -> getChars();
  }

String *SmiBinaryStringDefaultValue::toString() const {
  return new String(binaryString);
  }

