/*
 *  Copyright (C) 2003  Clive Butler
 *
 *  Author's email address: cebdev@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef SMIDEFAULTVALUE_H
#define SMIDEFAULTVALUE_H

#include <classlib/Object.hpp>
#include <classlib/String.hpp>

#include "SmiIdentifierList.hpp"

class SmiDefaultValue : public Object {
  public:
    SmiDefaultValue();
    virtual ~SmiDefaultValue();

    enum Type { INTEGER_VALUE, HEX_STRING_VALUE, BINARY_STRING_VALUE,
                IDENTIFIER_VALUE, IDLIST_VALUE, OID_VALUE, STRING_VALUE };

    virtual Type getType() const = 0;

  private:
    SmiDefaultValue(const SmiDefaultValue& other);
    SmiDefaultValue& operator=(const SmiDefaultValue& other);
  };

class SmiIdentifierListDefaultValue : public SmiDefaultValue {
  public:
    SmiIdentifierListDefaultValue(SmiIdentifierList *idlist);
    ~SmiIdentifierListDefaultValue();

    SmiDefaultValue::Type getType() const;

    SmiIdentifierList *getValue() const;

    String *toString() const;

  private:
    SmiIdentifierListDefaultValue(const SmiIdentifierListDefaultValue& other);
    SmiIdentifierListDefaultValue&
          operator=(const SmiIdentifierListDefaultValue& other);

    SmiIdentifierList *idlist;
  };

class SmiIdentifierDefaultValue : public SmiDefaultValue {
  public:
    SmiIdentifierDefaultValue(const char *string);
    ~SmiIdentifierDefaultValue();

    SmiDefaultValue::Type getType() const;

    const char *getValue() const;

    String *toString() const;

  private:
    SmiIdentifierDefaultValue(const SmiIdentifierDefaultValue& other);
    SmiIdentifierDefaultValue&
           operator=(const SmiIdentifierDefaultValue& other);

    String *value;
  };

class SmiStringDefaultValue : public SmiDefaultValue {
  public:
    SmiStringDefaultValue(const char *string);
    ~SmiStringDefaultValue();

    SmiDefaultValue::Type getType() const;

    const char *getValue() const;

    String *toString() const;

  private:
    SmiStringDefaultValue(const SmiStringDefaultValue& other);
    SmiStringDefaultValue& operator=(const SmiStringDefaultValue& other);

    String *value;
  };

class SmiIntegerDefaultValue : public SmiDefaultValue {
  public:
    SmiIntegerDefaultValue();
    SmiIntegerDefaultValue(const char *integerString);
    virtual ~SmiIntegerDefaultValue();

    virtual SmiDefaultValue::Type getType() const;

    long long getValue() const;

    virtual String *toString() const;

  protected:
    long long value;

  private:
    SmiIntegerDefaultValue(const SmiIntegerDefaultValue& other);
    SmiIntegerDefaultValue& operator=(const SmiIntegerDefaultValue& other);
  };

class SmiHexStringDefaultValue : public SmiDefaultValue {
  public:
    SmiHexStringDefaultValue(const char *hexString);
    ~SmiHexStringDefaultValue();

    SmiDefaultValue::Type getType() const;

    const char *getValue() const;

    String *toString() const;

  private:
    SmiHexStringDefaultValue(const SmiHexStringDefaultValue& other);
    SmiHexStringDefaultValue& operator=(const SmiHexStringDefaultValue& other);

    String *hexString;
  };

class SmiBinaryStringDefaultValue : public SmiDefaultValue {
  public:
    SmiBinaryStringDefaultValue(const char *binaryString);
    ~SmiBinaryStringDefaultValue();

    SmiDefaultValue::Type getType() const;

    const char *getValue() const;

    String *toString() const;

  private:
    SmiBinaryStringDefaultValue(const SmiBinaryStringDefaultValue& other);
    SmiBinaryStringDefaultValue&
            operator=(const SmiBinaryStringDefaultValue& other);

    String *binaryString;
  };

#endif
