/*
 *  Copyright (C) 2003  Clive Butler
 *
 *  Author's email address: cebdev@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <cstdio>
#include <cstring>

#include "SmiDefaultValueLoader.hpp"
#include "SmiIdentifierList.hpp"

SmiDefaultValueLoader::SmiDefaultValueLoader(const Xml& xml)
         : XmlLoader(xml), defval(0) {
  }

SmiDefaultValueLoader::~SmiDefaultValueLoader() {
  delete defval;
  }

static SmiIdentifierList *readIdentifierList(const Xml& xml,
                                             XmlElement idlistElement) {

  SmiIdentifierList *idlist = new SmiIdentifierList();

  int childCount = xml.getElementChildCount(idlistElement);
  for (int i = 0 ; i < childCount ; i++ ) {
    XmlElement child = xml.getElementChild(idlistElement,i);
    idlist -> add(xml.getElementContents(child));
    }

  return idlist;
  }

void SmiDefaultValueLoader::load(XmlElement defvalElement) {
  int childCount = xml.getElementChildCount(defvalElement);
  for (int i = 0 ; i < childCount ; i++ ) {
    XmlElement child = xml.getElementChild(defvalElement,i);
    const char *name = xml.getElementName(child);

    if (std::strcmp(name,"integer") == 0) {
      defval = new SmiIntegerDefaultValue(xml.getElementContents(child));
      }
    else if (std::strcmp(name,"identifier") == 0) {
      defval = new SmiIdentifierDefaultValue(xml.getElementContents(child));
      }
    else if (std::strcmp(name,"hstring") == 0) {
      defval = new SmiHexStringDefaultValue(xml.getElementContents(child));
      }
    else if (std::strcmp(name,"cstring") == 0) {
      defval = new SmiStringDefaultValue(xml.getElementContents(child));
      }
    else if (std::strcmp(name,"bstring") == 0) {
      defval =
          new SmiBinaryStringDefaultValue(xml.getElementContents(child));
      }
    else if (std::strcmp(name,"idlist") == 0) {
      defval = new SmiIdentifierListDefaultValue(readIdentifierList(xml,child));
      }
    else if (std::strcmp(name,"oid") == 0) {
      // TBD?
      }
    }
  }

//
// Return the loaded default value object, transferring ownership
// (responsibility to delete) to the caller
//
SmiDefaultValue *SmiDefaultValueLoader::takeDefaultValue() {
  SmiDefaultValue *returnedDefval = defval;
  defval = 0;
  return returnedDefval;
  }
