/*
 *  Copyright (C) 2003  Clive Butler
 *
 *  Author's email address: cebdev@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <cstdio>
#include <cstdlib>
#include <cstring>

#include "SmiDriver.hpp"
#include "SmiNotFoundException.hpp"
#include "SmiDbException.hpp"

SmiDriver::SmiDriver(const SmiDb *smiDb) : key(0), smiDb(smiDb) {
  }

SmiDriver::SmiDriver(const SmiDb *smiDb,const char *key) {
  this -> smiDb = smiDb;
  this -> key = new char[std::strlen(key) + 1];
  std::strcpy(this -> key,key);
  }

SmiDriver::~SmiDriver() {
  delete[] key;
  }

void SmiDriver::setKey(const char *key) {
  if (this -> key) delete[] key;
  this -> key = new char[std::strlen(key) + 1];
  std::strcpy(this -> key,key);
  }

void SmiDriver::load() {
  int length;

  void *record = smiDb -> fetch(key,&length);
  if (record == 0) {
    char errorText[256];

    std::sprintf(errorText,"Database record %s does not exist",key);
    THROW(new SmiNotFoundException(errorText),);
    }

  Xml xml;
  xml.loadXmlFromBuffer((const char *)record,length);
  if (xml.exception) {
    SmiDb::free(record);

    char errorText[256];
    std::sprintf(errorText,"XML parse error for database record %s",key);
    THROW(new SmiDbException(errorText),);
    }

  loaded(xml);

  SmiDb::free(record);
  }
