/*
 *  Copyright (C) 2003  Clive Butler
 *
 *  Author's email address: cebdev@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <cstdio>
#include <cstring>

#include "SmiIndexListLoader.hpp"
#include "SmiIndexItem.hpp"

SmiIndexListLoader::SmiIndexListLoader(const Xml& xml)
         : XmlLoader(xml), indices(0) {
  }

SmiIndexListLoader::~SmiIndexListLoader() {
  delete indices;
  }

void SmiIndexListLoader::load(XmlElement indicesElement) {
  indices = new SmiIndexList();

  if (xml.getElementAttributeCount(indicesElement) >= 1) {
    if (std::strcmp(xml.getElementAttributeValue(indicesElement,0),
                    "true") == 0) {
      indices -> setImplied(true);
      }
    }

  int childCount = xml.getElementChildCount(indicesElement);
  for (int i = 0 ; i < childCount ; i++ ) {
    XmlElement child = xml.getElementChild(indicesElement,i);
    const char *name = xml.getElementName(child);

    if (std::strcmp(name,"index") == 0) {
      SmiIndexItem *item = new SmiIndexItem();

      int attributeCount = xml.getElementAttributeCount(child);
      for (int a = 0 ; a < attributeCount ; a++ ) {
        const char *attributeName = xml.getElementAttributeName(child,a);
        if (strcmp(attributeName,"name") == 0)
          item -> setName(xml.getElementAttributeValue(child,a));
        else if (strcmp(attributeName,"module") == 0)
          item -> setModule(xml.getElementAttributeValue(child,a));
        // else OID and typeref TBD?
        }

      indices -> add(item);
      }
    }
  }

//
// Return the loaded index list object, transferring ownership
// (responsibility to delete) to the caller
//
SmiIndexList *SmiIndexListLoader::takeIndexList() {
  SmiIndexList *returnedIndexList = indices;
  indices = 0;
  return returnedIndexList;
  }
