/*
 *  Copyright (C) 2003  Clive Butler
 *
 *  Author's email address: cebdev@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef SMIMODULE_H
#define SMIMODULE_H

#include <classlib/Object.hpp>
#include <classlib/String.hpp>
#include <classlib/ObjectHashMap.hpp>

#include "SmiDb.hpp"
#include "SmiAssignmentInfoList.hpp"
#include "SmiAssignment.hpp"
#include "SmiTypedefInfoList.hpp"
#include "SmiTypedef.hpp"
#include "SmiImportsMap.hpp"

class SmiModule : public Object {
  public:
    SmiModule(const char *name);
    SmiModule(const SmiDb *smiDb,const char *name);
    ~SmiModule();

    const char *getName() const;
    SmiAssignmentInfoList *getAssignmentInfoList() const;
    const SmiAssignment *getAssignment(const char *name) const;
    SmiTypedefInfoList *getTypedefInfoList() const;
    void addTypedef(SmiTypedef *typedefinition);
    const SmiTypedef *getTypedef(const char *name) const;
    const char *getModuleImport(const char *name) const;
    void setLanguage(const char *languageString);

    String *toString() const;
    String *loadField(const char *key) const;

  private:
    SmiModule(const SmiModule& other);
    SmiModule& operator=(const SmiModule& other);

    const SmiDb *smiDb;
    String *name;
    mutable SmiAssignmentInfoList *assignmentInfoList;
    mutable ObjectHashMap *assignmentCache;
    mutable SmiTypedefInfoList *typedefInfoList;
    mutable ObjectHashMap *typedefCache;
    mutable SmiImportsMap *importsMap;
  };

#endif
