/*
 *  Copyright (C) 2003  Clive Butler
 *
 *  Author's email address: cebdev@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <cstdio>
#include <cstdlib>
#include <cstring>

#include <classlib/String.hpp>

#include "SmiModuleList.hpp"
#include "SmiModuleListDriver.hpp"

SmiModuleList::SmiModuleList(SmiDb *smiDb,int cacheMaximumCapacity)
        : smiDb(smiDb), cacheMaximumCapacity(cacheMaximumCapacity) {

  moduleCache = new ObjectHashMap();
  SmiModuleListDriver driver(this,smiDb);
  driver.load();
  }

//
// Free each entry in the list (the list entries are deleted as
// well, so the list ends up empty).
//
static void freeAll(List *list) {
  ListIterator it = list -> iterator();
  while (it.hasNext()) {
    const void *item = it.next();
    it.remove();
    std::free((void *)item);
    }
  }

SmiModuleList::~SmiModuleList() {
  freeAll(this);
  delete moduleCache;
  }

//
// Return a module definition, from the module cache if possible
//
SmiModule *SmiModuleList::getModule(const char *moduleName) {
  SmiModule *cachedModule;

  ObjectMapping mapping = moduleCache -> find(moduleName);
  if (mapping.value != 0) {
    cachedModule = (SmiModule *)mapping.value;
    }
  else {
    if (moduleCache -> size() > cacheMaximumCapacity) {
      }

    cachedModule = new SmiModule(smiDb,moduleName);
    moduleCache -> add(new String(moduleName),cachedModule);
    }

  return cachedModule;
  }

void SmiModuleList::add(const char *moduleName) {
  List::add(strdup(moduleName));
  }
