/*
 *  Copyright (C) 2003  Clive Butler
 *
 *  Author's email address: cebdev@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <cstdio>
#include <cstring>

#include "SmiNotificationTypeLoader.hpp"
#include "SmiValueReferenceListLoader.hpp"
#include "SmiModule.hpp"

SmiNotificationTypeLoader::SmiNotificationTypeLoader(const SmiModule *module,
                                                     const Xml& xml)
        : SmiAssignmentLoader(module,xml), notificationType(0) {
  }

SmiNotificationTypeLoader::~SmiNotificationTypeLoader() {
  delete notificationType;
  }

void SmiNotificationTypeLoader::load(XmlElement notificationElement) {
  notificationType = new SmiNotificationType(module,
                  xml.getElementAttributeValue(notificationElement,0));
  notificationType ->
           setStatus(xml.getElementAttributeValue(notificationElement,1));

  int childCount = xml.getElementChildCount(notificationElement);
  for (int i = 0 ; i < childCount ; i++ ) {
    XmlElement child = xml.getElementChild(notificationElement,i);
    const char *name = xml.getElementName(child);

    if (std::strcmp(name,"description") == 0) {
      //
      // The description text is in a separate database record,
      // but we only fetch it when someone asks for it.
      //
      const char *value = xml.getElementContents(child);
      if (value && *value) notificationType -> setDescription(value);
      }
    else if (std::strcmp(name,"oid") == 0) { // The SMI OID
      }
    else if (std::strcmp(name,"objects") == 0) {
      SmiValueReferenceListLoader loader(xml);
      loader.load(child);
      notificationType -> setObjectList(loader.takeValueReferenceList());
      }
    else if (std::strcmp(name,"reference") == 0) {
      const char *value = xml.getElementContents(child);
      notificationType -> setReference(value);
      }
    }
  }

//
// Return the loaded object type, transferring ownership to the caller
//
SmiAssignment *SmiNotificationTypeLoader::takeAssignment() {
  SmiAssignment *returnedAssignment = (SmiAssignment *)notificationType;
  notificationType = 0;
  return returnedAssignment;
  }
