/*
 *  Copyright (C) 2003  Clive Butler
 *
 *  Author's email address: cebdev@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <cstdio>

#include "SmiObjectIdentity.hpp"
#include "SmiModule.hpp"

SmiObjectIdentity::SmiObjectIdentity(const SmiModule *module,const char *name)
          : SmiAssignment(module,name),
            status(0), description(0), reference(0) {
  }

SmiObjectIdentity::~SmiObjectIdentity() {
  delete reference;
  delete description;
  delete status;
  }

SmiAssignmentInfo::Type SmiObjectIdentity::getType() const {
  return SmiAssignmentInfo::OBJECT_IDENTITY;
  }

void SmiObjectIdentity::setStatus(const char *status) {
  this -> status = new String(status);
  }

const char *SmiObjectIdentity::getStatus() const {
  if (status) return status -> getChars();
  return 0;
  }

void SmiObjectIdentity::setDescription(const char *description) {
  this -> description = new String(description);
  }

const char *SmiObjectIdentity::getDescription() const {
  if (! description) {
    //
    // The description has not yet been loaded from the database,
    // so fetch it now
    //
    char key[128];
    sprintf(key,"assignments/%s/description",getName());
    description = module -> loadField(key);
    if (! description) description = new String("");
    }

  return description -> getChars();
  }

void SmiObjectIdentity::setReference(const char *reference) {
  this -> reference = new String(reference);
  }

const char *SmiObjectIdentity::getReference() const {
  if (reference) return reference -> getChars();
  return 0;
  }
