/*
 *  Copyright (C) 2003  Clive Butler
 *
 *  Author's email address: cebdev@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <cstdio>
#include <cstring>

#include "SmiObjectIdentityLoader.hpp"
#include "SmiModule.hpp"

SmiObjectIdentityLoader::SmiObjectIdentityLoader(const SmiModule *module,
                                                 const Xml& xml)
        : SmiAssignmentLoader(module,xml), objectIdentity(0) {
  }

SmiObjectIdentityLoader::~SmiObjectIdentityLoader() {
  delete objectIdentity;
  }

void SmiObjectIdentityLoader::load(XmlElement objectElement) {
  objectIdentity = new SmiObjectIdentity(module,
                  xml.getElementAttributeValue(objectElement,0));
  if (xml.getElementAttributeCount(objectElement) > 1) {
    objectIdentity -> setStatus(xml.getElementAttributeValue(objectElement,1));
    }

  int childCount = xml.getElementChildCount(objectElement);
  for (int i = 0 ; i < childCount ; i++ ) {
    XmlElement child = xml.getElementChild(objectElement,i);
    const char *name = xml.getElementName(child);

    if (std::strcmp(name,"description") == 0) {
      //
      // The description text is in a separate database record,
      // but we only fetch it when someone asks for it.
      //
      const char *value = xml.getElementContents(child);
      if (value && *value) objectIdentity -> setDescription(value);
      }
    else if (std::strcmp(name,"oid") == 0) { // The SMI OID
      }
    else if (std::strcmp(name,"reference") == 0) {
      const char *value = xml.getElementContents(child);
      objectIdentity -> setReference(value);
      }
    }
  }

//
// Return the loaded object identity, transferring ownership to the caller
//
SmiAssignment *SmiObjectIdentityLoader::takeAssignment() {
  SmiAssignment *returnedAssignment = (SmiAssignment *)objectIdentity;
  objectIdentity = 0;
  return returnedAssignment;
  }
