/*
 *  Copyright (C) 2003  Clive Butler
 *
 *  Author's email address: cebdev@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <cstdio>
#include <cstring>

#include "SmiObjectTypeLoader.hpp"
#include "SmiSyntaxLoader.hpp"
#include "SmiDefaultValueLoader.hpp"
#include "SmiIndexListLoader.hpp"
#include "SmiValueReferenceLoader.hpp"
#include "SmiModule.hpp"

SmiObjectTypeLoader::SmiObjectTypeLoader(const SmiModule *module,const Xml& xml)
        : SmiAssignmentLoader(module,xml), objectType(0) {
  }

SmiObjectTypeLoader::~SmiObjectTypeLoader() {
  delete objectType;
  }

void SmiObjectTypeLoader::load(XmlElement objectElement) {
  objectType = new SmiObjectType(module,
                  xml.getElementAttributeValue(objectElement,0));
  objectType -> setStatus(xml.getElementAttributeValue(objectElement,1));

  int childCount = xml.getElementChildCount(objectElement);
  for (int i = 0 ; i < childCount ; i++ ) {
    XmlElement child = xml.getElementChild(objectElement,i);
    const char *name = xml.getElementName(child);

    if (std::strcmp(name,"syntax") == 0) {
      SmiSyntaxLoader loader(xml);
      loader.load(child);
      objectType -> setSyntax(loader.takeSyntax());
      }
    else if (std::strcmp(name,"access") == 0) {
      const char *value = xml.getElementContents(child);
      objectType -> setAccess(value);
      }
    else if (std::strcmp(name,"description") == 0) {
      //
      // The description text is in a separate database record,
      // but we only fetch it when someone asks for it.
      //
      const char *value = xml.getElementContents(child);
      if (value && *value) objectType -> setDescription(value);
      }
    else if (std::strcmp(name,"oid") == 0) { // The SMI OID
      }
    else if (std::strcmp(name,"defval") == 0) {
      SmiDefaultValueLoader loader(xml);
      loader.load(child);
      objectType -> setDefaultValue(loader.takeDefaultValue());
      }
    else if (std::strcmp(name,"indices") == 0) {
      SmiIndexListLoader loader(xml);
      loader.load(child);
      objectType -> setIndexList(loader.takeIndexList());
      }
    else if (std::strcmp(name,"augments") == 0) {
      SmiValueReferenceLoader loader(xml);
      loader.load(child);
      objectType -> setAugments(loader.takeValueReference());
      }
    else if (std::strcmp(name,"units") == 0) {
      const char *value = xml.getElementContents(child);
      objectType -> setUnits(value);
      }
    else if (std::strcmp(name,"reference") == 0) {
      const char *value = xml.getElementContents(child);
      objectType -> setReference(value);
      }
    }
  }

//
// Return the loaded object type, transferring ownership to the caller
//
SmiAssignment *SmiObjectTypeLoader::takeAssignment() {
  SmiAssignment *returnedAssignment = (SmiAssignment *)objectType;
  objectType = 0;
  return returnedAssignment;
  }
