/*
 *  Copyright (C) 2003  Clive Butler
 *
 *  Author's email address: cebdev@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <cstdio>
#include <cstdlib>
#include <cstring>

#include "SmiOid.hpp"

SmiOid::SmiOid() : length(0) {
  }

SmiOid::SmiOid(const SmiTypes::SubIdentifier *sids,int sidCount) {
  setValue(sids,sidCount);
  }

SmiOid::SmiOid(const char *oidString) {
  setValue(oidString);
  }

SmiOid::SmiOid(const SmiOidTreeNode *node) {
  setValue(node);
  }

SmiOid::~SmiOid() {
  }

void SmiOid::setValue(const SmiTypes::SubIdentifier *sids,int sidCount) {
  std::memcpy(sid,sids,sidCount * sizeof(SmiTypes::SubIdentifier));
  length = sidCount;
  }

void SmiOid::setValue(const char *oidString) {
  length = 0;
  while (*oidString) {
    sid[length++] = atoi(oidString);
    while (*oidString && (*oidString != '.')) oidString++ ;
    if (*oidString) oidString++ ;
    }
  }

void SmiOid::setValue(const SmiOidTreeNode *node) {
  length = 0;
  while (node -> getParent()) {
    sid[length++] = node -> getSid();
    node = node -> getParent();
    }

  SmiTypes::SubIdentifier tmp;
  for (int i = 0 ; i < length / 2 ; i++ ) {
    tmp = sid[i]; sid[i] = sid[length - 1 - i]; sid[length - 1 - i] = tmp;
    }
  }

void SmiOid::append(const SmiOid& other) {
  std::memcpy(&sid[length],other.sid,
              sizeof(SmiTypes::SubIdentifier) * other.length);
  length += other.length;
  }

SmiOid& SmiOid::operator=(const char *oidString) {
  setValue(oidString);
  return *this;
  }

SmiOid& SmiOid::operator=(const SmiOidTreeNode *node) {
  setValue(node);
  return *this;
  }

unsigned char SmiOid::getLength() const {
  return length;
  }

const SmiTypes::SubIdentifier *SmiOid::getSids() const {
  return sid;
  }

bool SmiOid::isUnder(const SmiOid *other) const {
  //
  // An OID is under another if it has the other OID as a prefix
  //
  if (length < other -> length) return 0;
  for (int i = 0 ; i < other -> length ; i++ ) {
    if (sid[i] != other -> sid[i]) return false;
    }

  return true;
  }

bool SmiOid::sameBeginning(const SmiOid *other,int prefixLength) const {
  if ((length < prefixLength) || (other -> length < prefixLength)) return false;

  for (int i = 0 ; i < prefixLength ; i++ ) {
    if (sid[i] != other -> sid[i]) return false;
    }

  return true;
  }

bool SmiOid::sameEnding(const SmiOid *other,int suffixLength) const {
  if ((length < suffixLength) || (other -> length < suffixLength)) return false;

  int position = length - suffixLength;
  int otherPosition = other -> length - suffixLength;
  while (suffixLength) {
    if (sid[position] != other -> sid[otherPosition]) return false;

    position++ ;
    otherPosition++ ;
    suffixLength-- ;
    }

  return true;
  }

String *SmiOid::toString() const {
  return oidToString(this);
  }

String *oidToString(const SmiOid *oid) {
  unsigned char length;
  length = oid -> getLength();
  const SmiTypes::SubIdentifier *sid = oid -> getSids();

  char buffer[1024]; // Greater than 128 * 6 (maximum length of OID)

  char *nextPtr = buffer;
  for (unsigned char i = 0 ; i < length ; i++ ) {
    std::sprintf(nextPtr,"%lu",sid[i]);
    while (*nextPtr) nextPtr++ ;
    if (i < (length - 1)) *nextPtr++ = '.';
    }
  *nextPtr = 0;

  return new String(buffer);
  }
