/*
 *  Copyright (C) 2003  Clive Butler
 *
 *  Author's email address: cebdev@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef SMIOID_H
#define SMIOID_H

#include <classlib/Object.hpp>
#include <classlib/String.hpp>

#include "SmiTypes.hpp"
#include "SmiOidTreeNode.hpp"

class SmiOid : public Object {
  public:
    SmiOid();
    SmiOid(const SmiTypes::SubIdentifier *sids,int sidCount);
    SmiOid(const char *oidString);
    SmiOid(const SmiOidTreeNode *node);
    ~SmiOid();

    SmiOid& operator=(const char *oidString);
    SmiOid& operator=(const SmiOidTreeNode *node);

    void setValue(const SmiTypes::SubIdentifier *sids,int sidCount);
    void setValue(const char *oidString);
    void setValue(const SmiOidTreeNode *node);

    void append(const SmiOid& other);

    unsigned char getLength() const;
    const SmiTypes::SubIdentifier *getSids() const;
    bool isUnder(const SmiOid *other) const;
    bool sameBeginning(const SmiOid *other,int prefixLength) const;
    bool sameEnding(const SmiOid *other,int suffixLength) const;
    String *toString() const;

  private:
    SmiTypes::SubIdentifier sid[128];
    unsigned char length;
  };

extern String *oidToString(const SmiOid *oid);

#endif
