/*
 *  Copyright (C) 2003  Clive Butler
 *
 *  Author's email address: cebdev@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef SMIOIDTREE_H
#define SMIOIDTREE_H

#include "SmiDb.hpp"
#include "SmiOid.hpp"
#include "SmiOidTreeNode.hpp"

class ObjectHashMap;

class SmiOidTree {
  public:
    SmiOidTree(SmiDb *smiDb);
    ~SmiOidTree();

    const SmiOidTreeNode *add(const char *name,const char *number);
    const SmiOidTreeNode *findDeepest(const SmiOid *oid) const ;
    const SmiOidTreeNode *findDeepestNamed(const SmiOidTreeNode *node) const ;
    const SmiOidTreeNode *findDeepestNamed(const SmiOid *oid) const ;

    const char *getName(const SmiOidTreeNode *node) const ;
    const SmiOidTreeNode *getNode(const char *name) const ;

    const SmiOidTreeNode *getRootNode() const;

  private:
    SmiOidTree(const SmiOidTree& other);
    SmiOidTree& operator=(const SmiOidTree& other);

    SmiDb *smiDb;
    SmiOidTreeNode *root;
    ObjectHashMap *oidToNameMap;
    ObjectHashMap *nameToOidMap;
  };

#endif
