/*
 *  Copyright (C) 2003  Clive Butler
 *
 *  Author's email address: cebdev@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <cstdio>
#include <cstring>
#include <cstdlib>

#include <classlib/ObjectList.hpp>

#include "SmiOidTreeNode.hpp"

#include "SmiOid.hpp"

SmiOidTreeNode::SmiOidTreeNode() : sid(0), parent(0), children(0) {
  }

SmiOidTreeNode::SmiOidTreeNode(SmiTypes::SubIdentifier sid,
                               SmiOidTreeNode *parent)
            : sid(sid), parent(parent), children(0) {
  }

SmiOidTreeNode::~SmiOidTreeNode() {
  delete children;
  }

SmiTypes::SubIdentifier SmiOidTreeNode::getSid() const {
  return sid;
  }

SmiOidTreeNode *SmiOidTreeNode::getParent() const {
  return parent;
  }

ObjectList *SmiOidTreeNode::getChildren() const {
  return children;
  }

SmiOidTreeNode *SmiOidTreeNode::addChild(SmiTypes::SubIdentifier sid) {

  if (children == 0) {
    children = new ObjectList();
    SmiOidTreeNode *child = new SmiOidTreeNode(sid,this);
    children -> add(child);
    return child;
    }

  ListIterator it = children -> iterator();
  while (it.hasNext()) {
    SmiOidTreeNode *thisChild = (SmiOidTreeNode *)it.next();

    if (thisChild -> getSid() == sid) return thisChild;
    if (thisChild -> getSid() > sid) {
      SmiOidTreeNode *child = new SmiOidTreeNode(sid,this);
      it.insert(child);
      return child;
      }
    }

  SmiOidTreeNode *child = new SmiOidTreeNode(sid,this);
  children -> add(child);
  return child;
  }
