/*
 *  Copyright (C) 2003  Clive Butler
 *
 *  Author's email address: cebdev@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include "SmiSyntax.hpp"
#include "SmiEnumeration.hpp"

#include <cstring>

SmiSyntax::SmiSyntax() : type(ST_Undetermined), module(0), name(0), tag(0),
                         typeref(0),subtype(0), enumerationList(0), sizes(0),
                         ranges(0), components(0) {
  }

SmiSyntax::~SmiSyntax() {
  delete module;
  delete name;
  delete tag;
  delete typeref;
  delete subtype;
  delete enumerationList;
  delete sizes;
  delete ranges;
  delete components;
  }

void SmiSyntax::setName(const char *name) {
  this -> name = new String(name);
  }

const char *SmiSyntax::getName() const {
  if (name) return name -> getChars();
  else return 0;
  }

SmiSyntax::Type SmiSyntax::getType() const {
  if ((type != ST_Undetermined) || (! typeref)) return type;

  const char *typeName = typeref -> getChars();
  if (std::strcmp(typeName,"INTEGER") == 0) type = ST_Integer;
  else
  if (std::strcmp(typeName,"Integer32") == 0) type = ST_Integer;
  else
  if (std::strcmp(typeName,"OCTET STRING") == 0) type = ST_OctetString;
  else
  if (std::strcmp(typeName,"Counter32") == 0) type = ST_Counter;
  else
  if (std::strcmp(typeName,"Unsigned32") == 0) type = ST_UnsignedInteger;
  else
  if (std::strcmp(typeName,"Gauge32") == 0) type = ST_Gauge;
  else
  if (std::strcmp(typeName,"TimeTicks") == 0) type = ST_TimeTicks;
  else
  if (std::strcmp(typeName,"IpAddress") == 0) type = ST_IpAddress;
  else
  if (std::strcmp(typeName,"OBJECT IDENTIFIER") == 0) {
    type = ST_ObjectIdentifier;
    }
  else
  if (std::strcmp(typeName,"Counter64") == 0) type = ST_Counter;
  else type = ST_NotBaseType;

  return type;
  }

void SmiSyntax::setModule(const char *module) {
  this -> module = new String(module);
  }

const char *SmiSyntax::getModule() const {
  if (module) return module -> getChars();
  else return 0;
  }

void SmiSyntax::setTag(SmiAsn1Tag *tag) {
  this -> tag = tag;
  }

SmiAsn1Tag *SmiSyntax::getTag() const {
  return tag;
  }

void SmiSyntax::setTyperef(const char *typeref) {
  this -> typeref = new String(typeref);
  }

const char *SmiSyntax::getTyperef() const {
  if (typeref) return typeref -> getChars();
  else return 0;
  }

void SmiSyntax::setSubtype(const char *subtype) {
  this -> subtype = new String(subtype);
  }

const char *SmiSyntax::getSubtype() const {
  if (subtype) return subtype -> getChars();
  else return 0;
  }

void SmiSyntax::setEnumerationList(SmiEnumerationList *enums) {
  this -> enumerationList = enums;
  }

SmiEnumerationList *SmiSyntax::getEnumerationList() const {
  return enumerationList;
  }

void SmiSyntax::setSizeList(SmiSizeList *sizes) {
  this -> sizes = sizes;
  }

SmiSizeList *SmiSyntax::getSizeList() const {
  return sizes;
  }

void SmiSyntax::setRangeList(SmiRangeList *ranges) {
  this -> ranges = ranges;
  }

SmiRangeList *SmiSyntax::getRangeList() const {
  return ranges;
  }

void SmiSyntax::setComponentList(SmiComponentList *components) {
  this -> components = components;
  }

SmiComponentList *SmiSyntax::getComponentList() const {
  return components;
  }

const char *SmiSyntax::findEnumeration(int number) const {
  if (! enumerationList) return 0;

  return enumerationList -> findEnumeration(number);
  }
