/*
 *  Copyright (C) 2003  Clive Butler
 *
 *  Author's email address: cebdev@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <cstdio>
#include <cstring>

#include "SmiTypedefLoader.hpp"
#include "SmiSyntaxLoader.hpp"

SmiTypedefLoader::SmiTypedefLoader(const Xml& xml)
              : XmlLoader(xml), typedefinition(0) {
  }

SmiTypedefLoader::~SmiTypedefLoader() {
  delete typedefinition;
  }

void SmiTypedefLoader::load(XmlElement typedefElement) {

  typedefinition = new SmiTypedef(
                         xml.getElementAttributeValue(typedefElement,0));
  if (xml.getElementAttributeCount(typedefElement) == 2) {
    typedefinition ->
          setStatus(xml.getElementAttributeValue(typedefElement,1));
    }

  int childCount = xml.getElementChildCount(typedefElement);
  for (int i = 0 ; i < childCount ; i++ ) {
    XmlElement child = xml.getElementChild(typedefElement,i);
    const char *name = xml.getElementName(child);

    if (std::strcmp(name,"syntax") == 0) {
      SmiSyntaxLoader *loader = new SmiSyntaxLoader(xml);
      loader -> load(child);
      typedefinition -> setSyntax(loader -> takeSyntax());
      delete loader;
      }
    else if (std::strcmp(name,"description") == 0) {
      const char *value = xml.getElementContents(child);
      if (value && *value) typedefinition -> setDescription(value);
      }
    else if (std::strcmp(name,"reference") == 0) {
      const char *value = xml.getElementContents(child);
      typedefinition -> setReference(value);
      }
    else if (std::strcmp(name,"format") == 0) {
      const char *value = xml.getElementContents(child);
      typedefinition -> setDisplayHint(value);
      }
    }
  }

SmiTypedef *SmiTypedefLoader::takeTypedef() {
  SmiTypedef *returnedTypedef = typedefinition;
  typedefinition = 0;
  return returnedTypedef;
  }
