/*
 *  Copyright (C) 2003  Clive Butler
 *
 *  Author's email address: cebdev@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <cstdio>
#include <cstdlib>
#include <cstring>

#include "SmiUtility.hpp"
#include "SmiTypes.hpp"

void SmiUtility::writeNumber(long long value,int places,int radix,
                             char *result,int resultMax) {
  if (value < 0) {
    *result++ = '-';
    resultMax-- ;

    value = -value;
    }

  if (value == 0) {
    *result++ = '0';
    *result = '\0';
    return;
    }

  char scratch[128];
  char *scratchPtr = scratch + sizeof(scratch) - 1;
  int length = 0;
  while (value) {
    int digit = value % radix;
    if (digit < 10) *scratchPtr-- = digit + '0';
    else *scratchPtr-- = (digit - 10) + 'a';
    length++ ;
    value = value / radix;
    }

  if (length <= places) {
    while (length < places) {
      *scratchPtr-- = '0'; length++ ;
      }
    *scratchPtr-- = '.'; length++ ;
    *scratchPtr-- = '0'; length++ ;
    }

  std::memcpy(result,scratchPtr + 1,length);
  result[length] = '\0';
  }

#if 1
void SmiUtility::writeUnsignedInteger(unsigned long value,int places,int radix,
                                      char *result,int resultMax) {
  writeNumber(value,places,radix,result,resultMax);
  }
#else
void SmiUtility::writeUnsignedInteger(unsigned long value,int places,int radix,
                                      char *result,int resultMax) {
  if (value == 0) {
    *result++ = '0';
    *result = '\0';
    return;
    }

  char scratch[64];
  char *scratchPtr = scratch + sizeof(scratch) - 1;
  int length = 0;
  while (value) {
    int digit = value % radix;
    if (digit < 10) *scratchPtr-- = digit + '0';
    else *scratchPtr-- = (digit - 10) + 'a';
    length++ ;
    value = value / radix;
    }

  if (length <= places) {
    while (length < places) {
      *scratchPtr-- = '0'; length++ ;
      }
    *scratchPtr-- = '.'; length++ ;
    *scratchPtr-- = '0'; length++ ;
    }

  std::memcpy(result,scratchPtr + 1,length);
  result[length] = '\0';
  }
#endif

#if 1
void SmiUtility::writeSignedInteger(long value,int places,int radix,
                                    char *result,int resultMax) {
  writeNumber(value,places,radix,result,resultMax);
  }
#else
void SmiUtility::writeSignedInteger(long value,int places,int radix,
                                    char *result,int resultMax) {
  if (value >= 0) writeUnsignedInteger(value,places,radix,result,resultMax);
  else {
    *result++ = '-';
    resultMax-- ;
    writeUnsignedInteger(-value,places,radix,result,resultMax);
    }
  }
#endif

String *SmiUtility::hintedUnsignedInteger(const char *hint,
                                          unsigned long value) {
  char scratch[64];

  switch (*hint++ ) {
    case 'd': {
      if (*hint++ == '-') {
        writeUnsignedInteger(value,atoi(hint),10,scratch,sizeof(scratch));
        }
      else {
        writeUnsignedInteger(value,0,10,scratch,sizeof(scratch));
        }
      break;
      }
    case 'x': {
      writeUnsignedInteger(value,0,16,scratch,sizeof(scratch));
      break;
      }
    case 'b': {
      writeUnsignedInteger(value,0,2,scratch,sizeof(scratch));
      break;
      }
    case 'o': {
      writeUnsignedInteger(value,0,8,scratch,sizeof(scratch));
      break;
      }
    }

  return new String(scratch);
  }

String *SmiUtility::hintedSignedInteger(const char *hint,long value) {
  char scratch[64];

  switch (*hint++ ) {
    case 'd': {
      if (*hint++ == '-') {
        writeSignedInteger(value,atoi(hint),10,scratch,sizeof(scratch));
        }
      else {
        writeSignedInteger(value,0,10,scratch,sizeof(scratch));
        }
      break;
      }
    case 'x': {
      writeSignedInteger(value,0,16,scratch,sizeof(scratch));
      break;
      }
    case 'b': {
      writeSignedInteger(value,0,2,scratch,sizeof(scratch));
      break;
      }
    case 'o': {
      writeSignedInteger(value,0,8,scratch,sizeof(scratch));
      break;
      }
    }

  return new String(scratch);
  }

